@@ -4,17 +4,24 @@ import os
 import re
 import sys
 import warnings
-from argparse import Action, SUPPRESS, _HelpAction, _SubParsersAction
+from argparse import SUPPRESS
+from argparse import Action as ArgparseAction
+from argparse import _HelpAction, _SubParsersAction
 from contextlib import contextmanager
 from contextvars import ContextVar
 from typing import Any, Dict, List, Optional, Tuple, Type, Union
 
+from ._common import parser_context
 from .loaders_dumpers import get_loader_exceptions, load_value
 from .namespace import Namespace, split_key, split_key_root
 from .optionals import FilesCompleterMethod, get_config_read_mode
 from .type_checking import ArgumentParser
 from .typing import path_type
 from .util import (
+    LoggerProperty,
+    NoneType,
+    ParserError,
+    Path,
     change_to_path_dir,
     default_config_option_help,
     get_typehint_origin,
@@ -22,14 +29,9 @@ from .util import (
     indent_text,
     is_subclass,
     iter_to_set_str,
-    lenient_check_context,
-    NoneType,
     parse_value_or_config,
-    ParserError,
-    Path,
 )
 
-
 __all__ = [
     'ActionConfigFile',
     'ActionYesNo',
@@ -38,6 +40,10 @@ __all__ = [
 ]
 
 
+class Action(LoggerProperty, ArgparseAction):
+    """Base for jsonargparse Action classes."""
+
+
 def _is_branch_key(parser, key: str) -> bool:
     root_key = split_key_root(key)[0]
     for action in filter_default_actions(parser._actions):
@@ -52,8 +58,8 @@ def _is_branch_key(parser, key: str) -> bool:
 def _find_action_and_subcommand(
     parser: 'ArgumentParser',
     dest: str,
-    exclude: Optional[Union[Type[Action], Tuple[Type[Action], ...]]] = None,
-) -> Tuple[Optional[Action], Optional[str]]:
+    exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
+) -> Tuple[Optional[ArgparseAction], Optional[str]]:
     """Finds an action in a parser given its destination key.
 
     Args:
@@ -89,16 +95,16 @@ def _find_action_and_subcommand(
 def _find_action(
     parser: 'ArgumentParser',
     dest: str,
-    exclude: Optional[Union[Type[Action], Tuple[Type[Action], ...]]] = None,
-) -> Optional[Action]:
+    exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
+) -> Optional[ArgparseAction]:
     return _find_action_and_subcommand(parser, dest, exclude=exclude)[0]
 
 
 def _find_parent_action_and_subcommand(
     parser: 'ArgumentParser',
     key: str,
-    exclude: Optional[Union[Type[Action], Tuple[Type[Action], ...]]] = None,
-) -> Tuple[Optional[Action], Optional[str]]:
+    exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
+) -> Tuple[Optional[ArgparseAction], Optional[str]]:
     action, subcommand = _find_action_and_subcommand(parser, key, exclude=exclude)
     if action is None and '.' in key:
         parts = split_key(key)
@@ -112,12 +118,12 @@ def _find_parent_action_and_subcommand(
 def _find_parent_action(
     parser: 'ArgumentParser',
     key: str,
-    exclude: Optional[Union[Type[Action], Tuple[Type[Action], ...]]] = None,
-) -> Optional[Action]:
+    exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
+) -> Optional[ArgparseAction]:
     return _find_parent_action_and_subcommand(parser, key, exclude=exclude)[0]
 
 
-def _is_action_value_list(action: Action) -> bool:
+def _is_action_value_list(action: ArgparseAction) -> bool:
     """Checks whether an action produces a list value.
 
     Args:
@@ -231,7 +237,7 @@ class _ActionPrintConfig(Action):
             metavar='\b[=flags]',
             help=(
                 'Print the configuration after applying all other arguments and exit. The optional '
-                'flags are one or more keywords separated by comma which modify the output. The '
+                'flags customizes the output and are one or more keywords separated by comma. The '
                 'supported flags are: comments, skip_default, skip_null.'
             ),
         )
@@ -267,7 +273,7 @@ class _ActionPrintConfig(Action):
             subparser = parser.print_config.pop('subparser')
             if key is not None:
                 cfg = cfg[key]
-            with lenient_check_context():
+            with parser_context(lenient_check=True):
                 sys.stdout.write(subparser.dump(cfg, **parser.print_config))
             delattr(parser, 'print_config')
             parser.exit()
@@ -301,6 +307,7 @@ class _ActionConfigLoad(Action):
                 Namespace({self.dest: namespace[self.dest]})
             )[self.dest]
         namespace[self.dest] = loaded_value
+        return None
 
     def _load_config(self, value, parser):
         try:
@@ -311,8 +318,8 @@ class _ActionConfigLoad(Action):
                 cfg = parser._apply_actions(cfg, parent_key=self.dest)
             return cfg
         except (TypeError,) + get_loader_exceptions() as ex:
-            str_ex = indent_text(str(ex))
-            raise TypeError(f'Parser key "{self.dest}": Unable to load config "{value}"\n- {str_ex}') from ex
+            str_ex = indent_text(f'- {ex}')
+            raise TypeError(f'Parser key "{self.dest}":\nUnable to load config {value!r}\n{str_ex}') from ex
 
     def check_type(self, value, parser):
         return self._load_config(value, parser)
@@ -357,7 +364,7 @@ class _ActionHelpClassPath(Action):
             kwargs['_baseclass'] = self._baseclass
             return type(self)(**kwargs)
         dest = re.sub('\\.help$', '',  self.dest)
-        self.print_help(args, self._baseclass, dest)
+        return self.print_help(args, self._baseclass, dest)
 
     def print_help(self, call_args, baseclass, dest):
         from .typehints import resolve_class_path_by_name
@@ -365,7 +372,7 @@ class _ActionHelpClassPath(Action):
         try:
             val_class = import_object(resolve_class_path_by_name(baseclass, value))
         except Exception as ex:
-            raise TypeError(f'{option_string}: {ex}')
+            raise TypeError(f'{option_string}: {ex}') from ex
         if get_typehint_origin(self._baseclass) == Union:
             baseclasses = self._baseclass.__args__
         else:
@@ -451,6 +458,7 @@ class ActionYesNo(Action):
             setattr(args[1], self.dest, not value)
         else:
             setattr(args[1], self.dest, value)
+        return None
 
     def _add_dest_prefix(self, prefix):
         self.dest = prefix+'.'+self.dest
@@ -569,7 +577,8 @@ def parent_parsers_context(key, parser):
 class _ActionSubCommands(_SubParsersAction):
     """Extension of argparse._SubParsersAction to modify subcommands functionality."""
 
-    _env_prefix: Union[bool, str] = True
+    parent_parser: 'ArgumentParser'
+    env_prefix: str
 
 
     def add_parser(self, name, **kwargs):
@@ -588,11 +597,12 @@ class _ActionSubCommands(_SubParsersAction):
             raise ValueError('Multiple levels of subcommands must be added in level order.')
 
         parser.prog = f'{self._prog_prefix} [options] {name}'
-        parser.env_prefix = f'{self._env_prefix}_{name}_'
+        parser.env_prefix = f'{self.env_prefix}{name}_'
         parser.default_env = self.parent_parser.default_env
         parser.parent_parser = self.parent_parser
         parser.parser_mode = self.parent_parser.parser_mode
         parser.error_handler = self.parent_parser.error_handler
+        parser.logger = self.parent_parser.logger
         parser.subcommand = name
 
         # create a pseudo-action to hold the choice help
@@ -784,6 +794,7 @@ class ActionPathList(Action, FilesCompleterMethod):
             kwargs['_rel'] = self._rel
             return ActionPathList(**kwargs)
         setattr(args[1], self.dest, self._check_type(args[2]))
+        return None
 
     def _check_type(self, value, cfg=None):
         if value == []:
