@@ -2,13 +2,13 @@
 
 import inspect
 from typing import Any, Callable, Dict, List, Optional, Type, Union
+
 from .actions import ActionConfigFile, _ActionPrintConfig, remove_actions
 from .core import ArgumentParser
 from .deprecated import deprecation_warning_cli_return_parser
 from .optionals import get_doc_short_description
 from .util import default_config_option_help
 
-
 __all__ = ['CLI']
 
 
@@ -18,6 +18,8 @@ def CLI(
     config_help: str = default_config_option_help,
     set_defaults: Optional[Dict[str, Any]] = None,
     as_positional: bool = True,
+    fail_untyped: bool = True,
+    parser_class: Type[ArgumentParser] = ArgumentParser,
     **kwargs
 ):
     """Simple creation of command line interfaces.
@@ -34,6 +36,8 @@ def CLI(
         config_help: Help string for config file option in help.
         set_defaults: Dictionary of values to override components defaults.
         as_positional: Whether to add required parameters as positional arguments.
+        fail_untyped: Whether to raise exception if a required parameter does not have a type.
+        parser_class: The ArgumentParser class to use.
         **kwargs: Used to instantiate :class:`.ArgumentParser`.
 
     Returns:
@@ -57,12 +61,12 @@ def CLI(
     elif not isinstance(components, list):
         components = [components]
 
-    parser = ArgumentParser(default_meta=False, **kwargs)
+    parser = parser_class(default_meta=False, **kwargs)
     parser.add_argument('--config', action=ActionConfigFile, help=config_help)
 
     if len(components) == 1:
         component = components[0]
-        _add_component_to_parser(component, parser, as_positional, config_help)
+        _add_component_to_parser(component, parser, as_positional, fail_untyped, config_help)
         if set_defaults is not None:
             parser.set_defaults(set_defaults)
         if return_parser:
@@ -76,10 +80,10 @@ def CLI(
     comp_dict = {c.__name__: c for c in components}
     for name, component in comp_dict.items():
         description = get_help_str(component, parser.logger)
-        subparser = ArgumentParser(description=description)
+        subparser = parser_class(description=description)
         subparser.add_argument('--config', action=ActionConfigFile, help=config_help)
         subcommands.add_subcommand(name, subparser, help=get_help_str(component, parser.logger))
-        added_args = _add_component_to_parser(component, subparser, as_positional, config_help)
+        added_args = _add_component_to_parser(component, subparser, as_positional, fail_untyped, config_help)
         if not added_args:
             remove_actions(subparser, (ActionConfigFile, _ActionPrintConfig))
 
@@ -102,8 +106,8 @@ def get_help_str(component, logger):
     return help_str
 
 
-def _add_component_to_parser(component, parser, as_positional, config_help):
-    kwargs = {'as_positional': as_positional, 'sub_configs': True}
+def _add_component_to_parser(component, parser, as_positional, fail_untyped, config_help):
+    kwargs = dict(as_positional=as_positional, fail_untyped=fail_untyped, sub_configs=True)
     if inspect.isfunction(component):
         added_args = parser.add_function_arguments(component, as_group=False, **kwargs)
         if not parser.description:
@@ -113,7 +117,7 @@ def _add_component_to_parser(component, parser, as_positional, config_help):
         subcommands = parser.add_subcommands(required=True)
         for key in [k for k, v in inspect.getmembers(component) if callable(v) and k[0] != '_']:
             description = get_help_str(getattr(component, key), parser.logger)
-            subparser = ArgumentParser(description=description)
+            subparser = type(parser)(description=description)
             subparser.add_argument('--config', action=ActionConfigFile, help=config_help)
             added_subargs = subparser.add_method_arguments(component, key, as_group=False, **kwargs)
             added_args += [f'{key}.{a}' for a in added_subargs]
