@@ -7,9 +7,39 @@ import logging
 import os
 import re
 import sys
-from typing import Any, Callable, Dict, List, NoReturn, Optional, Sequence, Set, Tuple, Type, Union
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    NoReturn,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    Union,
+)
 
-from .formatters import DefaultHelpFormatter, empty_help, formatter_context, get_env_var
+from ._common import lenient_check, parser_context
+from .actions import (
+    Action,
+    ActionConfigFile,
+    ActionParser,
+    _ActionConfigLoad,
+    _ActionPrintConfig,
+    _ActionSubCommands,
+    _find_action,
+    _find_action_and_subcommand,
+    _find_parent_action,
+    _find_parent_action_and_subcommand,
+    _is_action_value_list,
+    _is_branch_key,
+    filter_default_actions,
+    parent_parsers,
+    previous_config,
+)
+from .formatters import DefaultHelpFormatter, empty_help, get_env_var
 from .jsonnet import ActionJsonnet, ActionJsonnetExtVars
 from .jsonschema import ActionJsonSchema
 from .link_arguments import ActionLink, ArgumentLinking
@@ -17,32 +47,19 @@ from .loaders_dumpers import (
     check_valid_dump_format,
     dump_using_format,
     get_loader_exceptions,
-    loaders,
     load_value,
-    load_value_context,
+    loaders,
     set_omegaconf_loader,
     yaml_load,
 )
-from .namespace import is_meta_key, Namespace, patch_namespace, recreate_branches, split_key, split_key_leaf, strip_meta
-from .parameter_resolvers import ConditionalDefault
-from .signatures import is_pure_dataclass, SignatureArguments
-from .typehints import ActionTypeHint, is_subclass_spec
-from .typing import is_final_class
-from .actions import (
-    ActionParser,
-    ActionConfigFile,
-    _ActionSubCommands,
-    _ActionPrintConfig,
-    _ActionConfigLoad,
-    _is_branch_key,
-    _find_action,
-    _find_action_and_subcommand,
-    _find_parent_action,
-    _find_parent_action_and_subcommand,
-    _is_action_value_list,
-    filter_default_actions,
-    parent_parsers,
-    previous_config,
+from .namespace import (
+    Namespace,
+    is_meta_key,
+    patch_namespace,
+    recreate_branches,
+    split_key,
+    split_key_leaf,
+    strip_meta,
 )
 from .optionals import (
     argcomplete_autocomplete,
@@ -52,20 +69,21 @@ from .optionals import (
     import_fsspec,
     import_jsonnet,
 )
+from .parameter_resolvers import UnknownDefault
+from .signatures import SignatureArguments, is_pure_dataclass
+from .typehints import ActionTypeHint, is_subclass_spec
+from .typing import is_final_class
 from .util import (
+    ParserError,
+    Path,
     change_to_path_dir,
     get_private_kwargs,
     identity,
     is_subclass,
-    lenient_check,
-    lenient_check_context,
-    ParserError,
-    Path,
     return_parser_if_captured,
     usage_and_exit_error_handler,
 )
 
-
 __all__ = ['ActionsContainer', 'ArgumentParser']
 
 
@@ -115,9 +133,10 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
                     kwargs=kwargs,
                     enable_path=enable_path,
                     container=super(),
-                    logger=self.logger,
+                    logger=self._logger,
                 )
         action = super().add_argument(*args, **kwargs)
+        action.logger = self._logger  # type: ignore
         if isinstance(action, ActionConfigFile) and getattr(self, '_print_config', None) is not None:
             self.add_argument(self._print_config, action=_ActionPrintConfig)  # type: ignore
         if is_meta_key(action.dest):
@@ -150,7 +169,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
         parser = self.parser if hasattr(self, 'parser') else self
         if name is not None and name in parser.groups:
             raise ValueError(f'Group with name {name} already exists.')
-        group = _ArgumentGroup(parser, *args, logger=parser.logger, **kwargs)
+        group = _ArgumentGroup(parser, *args, logger=parser._logger, **kwargs)
         group.parser = parser
         parser._action_groups.append(group)
         if name is not None:
@@ -195,7 +214,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         are supported. Additionally it accepts:
 
         Args:
-            env_prefix: Prefix for environment variables.
+            env_prefix: Prefix for environment variables. ``True`` to derive from ``prog``.
             error_handler: Handler for parsing errors, if None raises :class:`.ParserError` exception.
             formatter_class: Class for printing help messages.
             logger: Configures the logger, see :class:`.LoggerProperty`.
@@ -233,18 +252,10 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         if caller not in {'jsonargparse', 'argcomplete'}:
             raise NotImplementedError('parse_known_args not implemented to dissuade its use, since typos in configs would go unnoticed.')
 
-        if args is None:
-            args = sys.argv[1:]
-        else:
-            args = list(args)
-            if not all(isinstance(a, str) for a in args):
-                self.error(f'All arguments are expected to be strings: {args}')
-        self.args = args
-
         namespace = argcomplete_namespace(caller, self, namespace)
 
         try:
-            with patch_namespace(), lenient_check_context(caller), ActionTypeHint.subclass_arg_context(self), load_value_context(self.parser_mode):
+            with patch_namespace(), parser_context(parent_parser=self, lenient_check=True), ActionTypeHint.subclass_arg_context(self):
                 namespace, args = self._parse_known_args(args, namespace)
         except (argparse.ArgumentError, ParserError) as ex:
             self.error(str(ex), ex)
@@ -271,7 +282,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         skip_required: bool = False,
         skip_subcommands: bool = False,
         fail_no_subcommand: bool = True,
-        log_message: Optional[str] = None,
     ) -> Namespace:
         """Common parsing code used by other parse methods.
 
@@ -284,7 +294,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             skip_required: Whether to skip check of required arguments.
             skip_subcommands: Whether to skip subcommand processing.
             fail_no_subcommand: Whether to fail if no subcommand given.
-            log_message: Message to log at INFO level after parsing.
 
         Returns:
             A config object with all parsed values.
@@ -296,12 +305,12 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             _ActionSubCommands.handle_subcommands(self, cfg, env=env, defaults=defaults, fail_no_subcommand=fail_no_subcommand)
 
         if defaults:
-            with lenient_check_context():
+            with parser_context(lenient_check=True):
                 ActionTypeHint.add_sub_defaults(self, cfg)
 
         _ActionPrintConfig.print_config_if_requested(self, cfg)
 
-        with load_value_context(self.parser_mode):
+        with parser_context(parent_parser=self):
             ActionLink.apply_parsing_links(self, cfg)
 
             if not skip_check and not lenient_check.get():
@@ -310,9 +319,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         if not (with_meta or (with_meta is None and self._default_meta)):
             cfg = strip_meta(cfg)
 
-        if log_message is not None:
-            self._logger.info(log_message)
-
         return cfg
 
 
@@ -329,7 +335,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         if env or (env is None and self._default_env):
             if environ is None:
                 environ = os.environ
-            with load_value_context(self.parser_mode):
+            with parser_context(load_value_mode=self.parser_mode):
                 cfg_env = self._load_env_vars(env=environ, defaults=defaults)
             cfg = self.merge_config(cfg_env, cfg)
 
@@ -363,10 +369,18 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
         """
-        skip_check = get_private_kwargs(kwargs, {'_skip_check': False})
+        skip_check = get_private_kwargs(kwargs, _skip_check=False)
         return_parser_if_captured(self)
         argcomplete_autocomplete(self)
 
+        if args is None:
+            args = sys.argv[1:]
+        else:
+            args = list(args)
+            if not all(isinstance(a, str) for a in args):
+                self.error(f'All arguments are expected to be strings: {args}')
+        self.args = args
+
         try:
             cfg = self._parse_defaults_and_environ(defaults, env)
             if namespace:
@@ -383,12 +397,12 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 defaults=defaults,
                 with_meta=with_meta,
                 skip_check=skip_check,
-                log_message='Parsed command line arguments.',
             )
 
         except (TypeError, KeyError) as ex:
             self.error(str(ex), ex)
 
+        self._logger.debug('Parsed command line arguments: %s', args)
         return parsed_cfg
 
 
@@ -415,15 +429,15 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
         """
-        skip_check, skip_required = get_private_kwargs(kwargs, {'_skip_check': False, '_skip_required': False})
+        skip_check, skip_required = get_private_kwargs(kwargs, _skip_check=False, _skip_required=False)
 
         try:
             cfg = self._parse_defaults_and_environ(defaults, env)
             if cfg_base:
                 cfg = self.merge_config(cfg_base, cfg)
 
-            cfg_obj = self._apply_actions(cfg_obj, prev_cfg=cfg)
-            cfg = self.merge_config(cfg_obj, cfg)
+            cfg_apply = self._apply_actions(cfg_obj, prev_cfg=cfg)
+            cfg = self.merge_config(cfg_apply, cfg)
 
             parsed_cfg = self._parse_common(
                 cfg=cfg,
@@ -432,12 +446,12 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 with_meta=with_meta,
                 skip_check=skip_check,
                 skip_required=skip_required,
-                log_message='Parsed object.',
             )
 
         except (TypeError, KeyError) as ex:
             self.error(str(ex), ex)
 
+        self._logger.debug('Parsed object: %s', cfg_obj)
         return parsed_cfg
 
 
@@ -494,7 +508,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
         """
-        skip_check, skip_subcommands = get_private_kwargs(kwargs, {'_skip_check': False, '_skip_subcommands': False})
+        skip_check, skip_subcommands = get_private_kwargs(kwargs, _skip_check=False, _skip_subcommands=False)
 
         try:
             cfg = self._parse_defaults_and_environ(defaults, env=True, environ=env)
@@ -506,12 +520,12 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 with_meta=with_meta,
                 skip_check=skip_check,
                 skip_subcommands=skip_subcommands,
-                log_message='Parsed environment variables.',
             )
 
         except (TypeError, KeyError) as ex:
             self.error(str(ex), ex)
 
+        self._logger.debug('Parsed environment variables')
         return parsed_cfg
 
 
@@ -552,8 +566,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 **kwargs,
             )
 
-        self._logger.info(f'Parsed {self.parser_mode} from path: {cfg_path}')
-
+        self._logger.debug('Parsed configuration from path: %s', cfg_path)
         return parsed_cfg
 
 
@@ -583,10 +596,10 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         Raises:
             ParserError: If there is a parsing error and error_handler=None.
         """
-        skip_check, fail_no_subcommand = get_private_kwargs(kwargs, {'_skip_check': False, '_fail_no_subcommand': True})
+        skip_check, fail_no_subcommand = get_private_kwargs(kwargs, _skip_check=False, _fail_no_subcommand=True)
 
         try:
-            with load_value_context(self.parser_mode):
+            with parser_context(load_value_mode=self.parser_mode):
                 cfg = self._load_config_parser_mode(cfg_str, cfg_path, ext_vars, previous_config.get())
 
             if defaults or env:
@@ -600,12 +613,12 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 with_meta=with_meta,
                 skip_check=skip_check,
                 fail_no_subcommand=fail_no_subcommand,
-                log_message=f'Parsed {self.parser_mode} string.',
             )
 
         except (TypeError, KeyError) as ex:
             self.error(str(ex), ex)
 
+        self._logger.debug('Parsed %s string: %s', self.parser_mode, cfg_str)
         return parsed_cfg
 
 
@@ -657,15 +670,15 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             **kwargs: All options that `argparse.ArgumentParser.add_subparsers` accepts.
         """
         if 'description' not in kwargs:
-            kwargs['description'] = 'For more details of each subcommand add it as argument followed by --help.'
-        with formatter_context(self):
+            kwargs['description'] = 'For more details of each subcommand, add it as an argument followed by --help.'
+        with parser_context(parent_parser=self):
             subcommands: _ActionSubCommands = super().add_subparsers(dest=dest, **kwargs)  # type: ignore
         if required:
             self.required_args.add(dest)
         subcommands._required = required  # type: ignore
         subcommands.required = False
-        subcommands.parent_parser = self  # type: ignore
-        subcommands._env_prefix = self.env_prefix
+        subcommands.parent_parser = self
+        subcommands.env_prefix = get_env_var(self)
         self._subcommands_action = subcommands
         return subcommands
 
@@ -702,7 +715,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         cfg = strip_meta(cfg)
         ActionLink.strip_link_target_keys(self, cfg)
 
-        with load_value_context(self.parser_mode):
+        with parser_context(load_value_mode=self.parser_mode):
             if not skip_check:
                 self.check_config(cfg)
 
@@ -717,7 +730,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 self._dump_cleanup_actions(defaults, self._actions, {'skip_check': True, 'skip_none': skip_none})
                 self._dump_delete_default_entries(cfg_dict, defaults.as_dict())
 
-        with formatter_context(self):
+        with parser_context(parent_parser=self):
             return dump_using_format(self, cfg_dict, 'yaml_comments' if yaml_comments else format)
 
 
@@ -736,7 +749,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             elif isinstance(action, ActionTypeHint):
                 value = cfg.get(action_dest)
                 if value is not None:
-                    value = action.serialize(value, dump_kwargs=dump_kwargs)
+                    with parser_context(parent_parser=self):
+                        value = action.serialize(value, dump_kwargs=dump_kwargs)
                     cfg.update(value, action_dest)
 
 
@@ -748,7 +762,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 class_object_val = None
                 if is_subclass_spec(val):
                     if val['class_path'] != default.get('class_path'):
-                        parser = ActionTypeHint.get_class_parser(val['class_path'])
+                        with parser_context(parent_parser=self):
+                            parser = ActionTypeHint.get_class_parser(val['class_path'])
                         default = {'init_args': parser.get_defaults().as_dict()}
                     class_object_val = val
                     val = val.get('init_args')
@@ -820,7 +835,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             ActionLink.strip_link_target_keys(self, cfg)
 
             if not skip_check:
-                with load_value_context(self.parser_mode):
+                with parser_context(load_value_mode=self.parser_mode):
                     self.check_config(strip_meta(cfg), branch=branch)
 
             def save_paths(cfg):
@@ -849,7 +864,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                             f.write(val.get_content())
                         cfg[key] = type(val)(str(val_path))
 
-            with change_to_path_dir(path_fc), formatter_context(self):
+            with change_to_path_dir(path_fc), parser_context(parent_parser=self):
                 save_paths(cfg)
             dump_kwargs['skip_check'] = True
             with open(path_fc(), 'w') as f:
@@ -945,15 +960,15 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             if (
                 action.default != argparse.SUPPRESS and
                 action.dest != argparse.SUPPRESS and
-                not isinstance(action.default, ConditionalDefault)
+                not isinstance(action.default, UnknownDefault)
             ):
                 cfg[action.dest] = recreate_branches(action.default)
 
-        self._logger.info('Loaded default values from parser.')
+        self._logger.debug('Loaded default values from parser')
 
         default_config_files = self._get_default_config_files()
         for key, default_config_file in default_config_files:
-            with change_to_path_dir(default_config_file), load_value_context(self.parser_mode):
+            with change_to_path_dir(default_config_file), parser_context(parent_parser=self):
                 cfg_file = self._load_config_parser_mode(default_config_file.get_content())
                 if key is not None:
                     cfg_file = cfg_file.get(key)
@@ -977,7 +992,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 cfg['__default_config__'] = [meta, default_config_file]
             else:
                 cfg['__default_config__'] = default_config_file
-            self._logger.info(f'Parsed configuration from default path: {default_config_file}')
+            self._logger.debug('Parsed default configuration from path: %s', default_config_file)
 
         ActionTypeHint.add_sub_defaults(self, cfg)
 
@@ -1065,7 +1080,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         try:
             if not skip_required and not lenient_check.get():
                 check_required(cfg, self)
-            with load_value_context(self.parser_mode):
+            with parser_context(load_value_mode=self.parser_mode):
                 check_values(cfg)
         except (TypeError, KeyError) as ex:
             prefix = 'Configuration check failed :: '
@@ -1114,10 +1129,10 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                     pass
                 else:
                     if value is not None:
-                        with load_value_context(self.parser_mode):
+                        with parser_context(parent_parser=self):
                             parent[key] = component.instantiate_classes(value)
             else:
-                with load_value_context(self.parser_mode):
+                with parser_context(load_value_mode=self.parser_mode):
                     component.instantiate_class(component, cfg)
 
         ActionLink.apply_instantiation_links(self, cfg, order=order)
@@ -1184,13 +1199,13 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 note = f'tried getting defaults considering default_config_files but failed due to: {ex}'
             group = self._default_config_files_group
             group.description = f'{self._default_config_files}, Note: {note}'
-        with formatter_context(self, defaults):
+        with parser_context(parent_parser=self, defaults_cache=defaults):
             help_str = super().format_help()
         return help_str
 
 
     def print_usage(self, file=None):
-        with formatter_context(self):
+        with parser_context(parent_parser=self):
             return super().print_usage(file)
 
 
@@ -1199,6 +1214,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         cfg: Union[Namespace, Dict[str, Any]],
         parent_key: str = '',
         prev_cfg: Optional[Namespace] = None,
+        skip_fn: Optional[Callable[[Any], bool]] = None,
     ) -> Namespace:
         """Runs _check_value_key on actions present in config."""
         if isinstance(cfg, dict):
@@ -1243,7 +1259,9 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
             action_dest = action.dest if subcommand is None else subcommand+'.'+action.dest
             value = cfg[action_dest]
-            with lenient_check_context(), load_value_context(self.parser_mode):
+            if skip_fn and skip_fn(value):
+                continue
+            with parser_context(parent_parser=self, lenient_check=True):
                 value = self._check_value_key(action, value, action_dest, prev_cfg)
             if isinstance(action, _ActionConfigLoad):
                 config_keys.add(action_dest)
@@ -1265,7 +1283,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             A new object with the merged configuration.
         """
         cfg = cfg_to.clone()
-        ActionTypeHint.discard_init_args_on_class_path_change(self, cfg, cfg_from)
+        with parser_context(parent_parser=self):
+            ActionTypeHint.discard_init_args_on_class_path_change(self, cfg, cfg_from)
         cfg.update(cfg_from)
         ActionTypeHint.apply_appends(self, cfg)
         return cfg
@@ -1294,7 +1313,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             if isinstance(value, str):
                 value = action.check_type(value, self)
         elif hasattr(action, '_check_type'):
-            value = action._check_type(value, cfg=cfg)
+            with parser_context(parent_parser=self):
+                value = action._check_type(value, cfg=cfg)
         elif action.type is not None:
             try:
                 if action.nargs in {None, '?'} or action.nargs == 0:
@@ -1406,7 +1426,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
 
     @default_meta.setter
-    def default_meta(self, default_meta:bool):
+    def default_meta(self, default_meta: bool):
         if isinstance(default_meta, bool):
             self._default_meta = default_meta
         else:
@@ -1423,22 +1443,23 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         Raises:
             ValueError: If an invalid value is given.
         """
-        return self._env_prefix  # type: ignore
+        return self._env_prefix
 
 
     @env_prefix.setter
     def env_prefix(self, env_prefix: Union[bool, str]):
-        if env_prefix is False:
-            self._env_prefix = None
-        elif env_prefix in {True, None}:
-            if env_prefix is None:
-                from .deprecated import deprecation_warning, env_prefix_property_none_message
-                deprecation_warning(ArgumentParser, env_prefix_property_none_message)
-            self._env_prefix = os.path.splitext(self.prog)[0]
-        elif isinstance(env_prefix, str):
-            self._env_prefix = env_prefix
-        else:
-            raise ValueError('env_prefix has to be a string or None.')
+        if env_prefix is None:
+            from .deprecated import (
+                deprecation_warning,
+                env_prefix_property_none_message,
+            )
+            deprecation_warning(ArgumentParser, env_prefix_property_none_message)
+            env_prefix = False
+        elif env_prefix is True:
+            env_prefix = os.path.splitext(self.prog)[0]
+        elif not isinstance(env_prefix, (bool, str)):
+            raise ValueError('env_prefix must be a string or a boolean.')
+        self._env_prefix = env_prefix
 
 
     @property
@@ -1488,7 +1509,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         self._dump_header = dump_header
 
 
-from .deprecated import parse_as_dict_patch, instantiate_subclasses_patch
+from .deprecated import instantiate_subclasses_patch, parse_as_dict_patch
+
 instantiate_subclasses_patch()
 if 'SPHINX_BUILD' not in os.environ:
     parse_as_dict_patch()
