@@ -2,25 +2,33 @@
 
 import inspect
 import re
-from argparse import Action, SUPPRESS
+from argparse import SUPPRESS
+from argparse import Action as ArgparseAction
 from collections import defaultdict
 from contextlib import contextmanager
 from contextvars import ContextVar
 from typing import Any, Callable, List, Optional, Tuple, Type, Union
-from .actions import _ActionConfigLoad, _ActionSubCommands, ActionConfigFile, filter_default_actions, _find_parent_action
+
+from .actions import (
+    Action,
+    ActionConfigFile,
+    _ActionConfigLoad,
+    _ActionSubCommands,
+    _find_parent_action,
+    filter_default_actions,
+)
 from .namespace import Namespace, split_key_leaf
 from .type_checking import ArgumentParser, _ArgumentGroup
 
-
 __all__ = ['ArgumentLinking']
 
 
 def find_parent_or_child_actions(
     parser: 'ArgumentParser',
     key: str,
-    exclude: Optional[Union[Type[Action], Tuple[Type[Action], ...]]] = None,
-) -> Optional[List[Action]]:
-    found: List[Action] = []
+    exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
+) -> Optional[List[ArgparseAction]]:
+    found: List[ArgparseAction] = []
     action = _find_parent_action(parser, key, exclude=exclude)
     if action is not None:
         found = [action]
@@ -36,8 +44,8 @@ def find_parent_or_child_actions(
 def find_subclass_action_or_class_group(
     parser: 'ArgumentParser',
     key: str,
-    exclude: Optional[Union[Type[Action], Tuple[Type[Action], ...]]] = None,
-) -> Optional[Union[Action, '_ArgumentGroup']]:
+    exclude: Optional[Union[Type[ArgparseAction], Tuple[Type[ArgparseAction], ...]]] = None,
+) -> Optional[Union[ArgparseAction, '_ArgumentGroup']]:
     from .typehints import ActionTypeHint
     action = _find_parent_action(parser, key, exclude=exclude)
     if ActionTypeHint.is_subclass_typehint(action):
@@ -103,17 +111,12 @@ class ActionLink(Action):
         compute_fn: Optional[Callable] = None,
         apply_on: str = 'parse',
     ):
+        if not hasattr(parser, '_links_group'):
+            parser._links_group = parser.add_argument_group('Linked arguments')
         self.parser = parser
-
-        # Set and check apply_on
         self.apply_on = apply_on
-        if apply_on not in {'parse', 'instantiate'}:
-            raise ValueError("apply_on must be 'parse' or 'instantiate'.")
-
-        # Set and check compute function
         self.compute_fn = compute_fn
-        if compute_fn is None and not isinstance(source, str):
-            raise ValueError('Multiple source keys requires a compute function.')
+        self._initial_input_checks(source, target)
 
         # Set and check source actions or group
         exclude = (ActionLink, _ActionConfigLoad, _ActionSubCommands, ActionConfigFile)
@@ -134,11 +137,12 @@ class ActionLink(Action):
         assert self.target[1] is not None
 
         from .typehints import ActionTypeHint
-        is_target_subclass = ActionTypeHint.is_subclass_typehint(self.target[1])
+        is_target_subclass = ActionTypeHint.is_subclass_typehint(self.target[1], all_subtypes=False)
         valid_target_init_arg = is_target_subclass and target.startswith(self.target[1].dest+'.init_args.')
         valid_target_leaf = self.target[1].dest == target
-        if not (valid_target_leaf or valid_target_init_arg):
-            raise ValueError(f'Target key "{target}" must be for an individual argument.')
+        if not valid_target_leaf and is_target_subclass and not valid_target_init_arg:
+            prefix = self.target[1].dest+'.init_args.'
+            raise ValueError(f'Target key expected to start with "{prefix}", got "{target}".')
 
         # Replace target action with link action
         if not is_target_subclass or valid_target_leaf:
@@ -163,8 +167,6 @@ class ActionLink(Action):
             sub_add_kwargs['linked_targets'].add(subtarget)
 
         # Add link action to group to show in help
-        if not hasattr(parser, '_links_group'):
-            parser._links_group = parser.add_argument_group('Linked arguments')
         parser._links_group._group_actions.append(self)
 
         # Check instantiation link does not create cycle
@@ -198,6 +200,31 @@ class ActionLink(Action):
             help=help_str,
         )
 
+    def _initial_input_checks(self, source, target):
+        # Check apply_on
+        if self.apply_on not in {'parse', 'instantiate'}:
+            raise ValueError("apply_on must be 'parse' or 'instantiate'.")
+
+        # Check compute function
+        if self.compute_fn is None and not isinstance(source, str):
+            raise ValueError('Multiple source keys requires a compute function.')
+
+        if self.apply_on == 'parse':
+            # Check source
+            link_actions = self.parser._links_group._group_actions
+            existing_targets = {a.target[0] for a in link_actions}
+            for src in [source] if isinstance(source, str) else source:
+                if src in existing_targets:
+                    raise ValueError(f'Source "{src}" not allowed since it is the target of another link.')
+            # Check target
+            existing_sources = {
+                s[0]
+                for a in link_actions
+                for s in a.source if a.apply_on == 'parse'
+            }
+            if target in existing_sources:
+                raise ValueError(f'Target "{target}" not allowed since it is the source of another link.')
+
     def __call__(self, *args, **kwargs):
         source = ', '.join(s[0] for s in self.source)
         raise TypeError(f'Linked "{self.target[0]}" must be given via "{source}".')
@@ -296,7 +323,7 @@ class ActionLink(Action):
     def set_target_value(action: 'ActionLink', value: Any, cfg: Namespace, logger) -> None:
         target_key, target_action = action.target
         from .typehints import ActionTypeHint
-        if ActionTypeHint.is_subclass_typehint(target_action):
+        if ActionTypeHint.is_subclass_typehint(target_action, all_subtypes=False):
             if target_key == target_action.dest:  # type: ignore
                 target_action._check_type(value)  # type: ignore
             elif target_key not in cfg:
