@@ -3,12 +3,10 @@
 import inspect
 import locale
 import os
-import typing
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 from importlib.util import find_spec
 from typing import Optional
 
-
 __all__ = [
     'get_config_read_mode',
     'set_config_read_mode',
@@ -17,9 +15,10 @@ __all__ = [
 
 
 typing_extensions_support = find_spec('typing_extensions') is not None
+typeshed_client_support = find_spec('typeshed_client') is not None
 jsonschema_support = find_spec('jsonschema') is not None
 jsonnet_support = find_spec('_jsonnet') is not None
-url_support = False if any(find_spec(x) is None for x in ['validators', 'requests']) else True
+url_support = find_spec('requests') is not None
 docstring_parser_support = find_spec('docstring_parser') is not None
 argcomplete_support = find_spec('argcomplete') is not None
 fsspec_support = find_spec('fsspec') is not None
@@ -38,7 +37,7 @@ def typing_extensions_import(name):
     if typing_extensions_support:
         return getattr(__import__('typing_extensions'), name)
     else:
-        return getattr(typing, name, False)
+        return getattr(__import__('typing'), name, False)
 
 
 def is_compatible_final(final) -> bool:
@@ -61,6 +60,14 @@ if not final or not is_compatible_final(final) or 'SPHINX_BUILD' in os.environ:
         return cls
 
 
+def import_typeshed_client():
+    if typeshed_client_support:
+        import typeshed_client
+        return typeshed_client
+    else:
+        return __import__('argparse').Namespace(ImportedInfo=object, ModulePath=object, Resolver=object)
+
+
 class UndefinedException(Exception):
     pass
 
@@ -81,8 +88,7 @@ def missing_package_raise(package, importer):
 def import_jsonschema(importer):
     with missing_package_raise('jsonschema', importer):
         import jsonschema
-        from jsonschema import Draft7Validator as jsonvalidator
-    return jsonschema, jsonvalidator
+    return jsonschema, jsonschema.Draft7Validator
 
 
 def import_jsonnet(importer):
@@ -91,12 +97,6 @@ def import_jsonnet(importer):
     return _jsonnet
 
 
-def import_url_validator(importer):
-    with missing_package_raise('validators', importer):
-        from validators.url import url as url_validator
-    return url_validator
-
-
 def import_requests(importer):
     with missing_package_raise('requests', importer):
         import requests
@@ -144,15 +144,14 @@ def set_config_read_mode(
         fsspec_enabled: Whether to read config files from fsspec supported file systems.
     """
     imports = {
-        'u': [import_url_validator, import_requests],
-        's': [import_fsspec],
+        'u': import_requests,
+        's': import_fsspec,
     }
 
     def update_mode(flag, enabled):
         global _config_read_mode
         if enabled:
-            for import_func in imports[flag]:
-                import_func('set_config_read_mode')
+            imports[flag]('set_config_read_mode')
             if flag not in _config_read_mode:
                 _config_read_mode = _config_read_mode.replace('f', 'f'+flag)
         else:
@@ -204,6 +203,7 @@ def parse_docstring(component, params=False, logger=None):
     except (ValueError, dp.ParseError) as ex:
         if logger:
             logger.debug(f'Failed parsing docstring for {component}: {ex}')
+    return None
 
 
 def parse_docs(component, parent, logger):
@@ -231,6 +231,7 @@ def get_doc_short_description(function_or_class, method_name=None, logger=None):
         docstring = parse_docstring(component, params=False, logger=logger)
         if docstring:
             return docstring.short_description
+    return None
 
 
 def get_files_completer():
@@ -247,9 +248,9 @@ class FilesCompleterMethod:
 
 def argcomplete_autocomplete(parser):
     if argcomplete_support:
-        argcomplete = import_argcomplete('parse_args')
-        from .loaders_dumpers import load_value_context
-        with load_value_context(parser.parser_mode):
+        argcomplete = import_argcomplete('argcomplete_autocomplete')
+        from ._common import parser_context
+        with parser_context(load_value_mode=parser.parser_mode):
             argcomplete.autocomplete(parser)
 
 
@@ -264,11 +265,9 @@ def argcomplete_warn_redraw_prompt(prefix, message):
     argcomplete = import_argcomplete('argcomplete_warn_redraw_prompt')
     if prefix != '':
         argcomplete.warn(message)
-        try:
+        with suppress(ValueError):
             shell_pid = int(os.popen('ps -p %d -oppid=' % os.getppid()).read().strip())
             os.kill(shell_pid, 28)
-        except ValueError:
-            pass
     _ = '_' if locale.getlocale()[1] != 'UTF-8' else '\xa0'
     return [_+message.replace(' ', _), '']
 
@@ -276,6 +275,7 @@ def argcomplete_warn_redraw_prompt(prefix, message):
 def get_omegaconf_loader():
     """Returns a yaml loader function based on OmegaConf which supports variable interpolation."""
     import io
+
     from .loaders_dumpers import yaml_load
     with missing_package_raise('omegaconf', 'get_omegaconf_loader'):
         from omegaconf import OmegaConf
