@@ -8,15 +8,16 @@ from contextvars import ContextVar
 from copy import deepcopy
 from dataclasses import dataclass
 from functools import partial
-from typing import Any, Callable, List, Optional, Tuple, Type, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
+from ._stubs_resolver import get_stub_types
 from .optionals import parse_docs
 from .util import (
     ClassFromFunctionBase,
+    LoggerProperty,
     get_import_path,
     is_subclass,
     iter_to_set_str,
-    LoggerProperty,
     parse_logger,
     unique,
 )
@@ -46,12 +47,23 @@ class SourceNotAvailable(Exception):
     "Raised when the source code for some component is not available."
 
 
-class ConditionalDefault(list):
-    def __repr__(self):
-        return iter_to_set_str(self, sep=', ')
+class UnknownDefault:
+    def __init__(self, resolver: str, data: Any = inspect._empty) -> None:
+        self.resolver = resolver
+        self.data = data
+    def __repr__(self) -> str:
+        value = f"{type(self).__name__.replace('Default', '')}<{self.resolver}>"
+        if self.data != inspect._empty:
+            value = f'{value} {self.data}'
+        return value
 
 
-def get_parameter_origins(component, parent) -> str:
+class ConditionalDefault(UnknownDefault):
+    def __init__(self, resolver: str, data: Any) -> None:
+        super().__init__(resolver, iter_to_set_str(data, sep=', '))
+
+
+def get_parameter_origins(component, parent) -> Optional[str]:
     if isinstance(component, tuple):
         assert parent is None or len(component) == len(parent)
         return iter_to_set_str(
@@ -216,9 +228,12 @@ def ast_is_attr_assign(node, container):
 
 def ast_get_call_kwarg_with_value(node, value):
     value_dump = ast.dump(value)
+    kwarg = None
     for arg in node.keywords:
         if isinstance(getattr(arg, 'value', None), ast.AST) and ast.dump(arg.value) == value_dump:
-            return arg
+            kwarg = arg
+            break
+    return kwarg
 
 
 def ast_get_call_positional_indexes(node):
@@ -242,10 +257,10 @@ def get_arg_kind_index(params, kind):
 
 
 def get_signature_parameters_and_indexes(component, parent, logger):
+    signature_source = component
     if is_classmethod(parent, component):
-        params = list(inspect.signature(component.__func__).parameters.values())
-    else:
-        params = list(inspect.signature(component).parameters.values())
+        signature_source = component.__func__
+    params = list(inspect.signature(signature_source).parameters.values())
     if parent:
         params = params[1:]
     args_idx = get_arg_kind_index(params, kinds.VAR_POSITIONAL)
@@ -261,7 +276,26 @@ def get_signature_parameters_and_indexes(component, parent, logger):
             component=component,
             **{a: getattr(param, a) for a in parameter_attributes},
         )
-    return params, args_idx, kwargs_idx, doc_params
+    stubs = get_stub_types(params, signature_source, parent, logger)
+    return params, args_idx, kwargs_idx, doc_params, stubs
+
+
+def add_stub_types(stubs: Optional[Dict[str, Any]], params: ParamList, component) -> None:
+    if not stubs:
+        return
+    for param in params:
+        if param.annotation == inspect._empty and param.name in stubs:
+            param.annotation = stubs[param.name]
+    known_params = {p.name for p in params}
+    for name, stub in stubs.items():
+        if name not in known_params:
+            params.append(ParamData(
+                name=name,
+                annotation=stub,
+                default=UnknownDefault('stubs-resolver'),
+                kind=kinds.KEYWORD_ONLY,
+                component=component,
+            ))
 
 
 ast_literals = {
@@ -294,6 +328,9 @@ def get_kwargs_pop_or_get_parameter(node, component, parent, doc_params, log_deb
 
 
 def is_param_subclass_instance_default(param: ParamData) -> bool:
+    from .signatures import is_pure_dataclass
+    if is_pure_dataclass(type(param.default)):
+        return False
     from .typehints import ActionTypeHint, get_subclass_types
     class_types = get_subclass_types(param.annotation)
     return (
@@ -355,7 +392,10 @@ def group_parameters(params_list: List[ParamList]) -> ParamList:
             gparam.parent = tuple(p.parent for p in params)
             gparam.component = tuple(p.component for p in params)
             gparam.origin = tuple(p.origin for p in params)
-            gparam.default = ConditionalDefault((p.default for p in params) if len(defaults) > 1 else defaults)
+            gparam.default = ConditionalDefault(
+                'ast-resolver',
+                (p.default for p in params) if len(defaults) > 1 else defaults,
+            )
             if len(types) > 1:
                 gparam.annotation = Union[tuple(types)] if types else inspect._empty
         docs = [p.doc for p in params if p.doc]
@@ -577,7 +617,7 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
                 subclass_types = get_subclass_types(param.annotation)
                 if not (class_type and subclass_types and is_subclass(class_type, subclass_types)):
                     continue
-                subclass_spec = dict(class_path=get_import_path(class_type), init_args=dict())
+                subclass_spec: dict = dict(class_path=get_import_path(class_type), init_args=dict())
                 for kwarg in node.keywords:
                     if kwarg.arg and ast_is_constant(kwarg.value):
                         subclass_spec['init_args'][kwarg.arg] = ast_get_constant_value(kwarg.value)
@@ -613,14 +653,15 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
         if args_name:
             values_to_find[args_name] = ast_variable_load(args_name)
         if kwargs_name:
-            values_to_find[kwargs_name] = kwargs_value = ast_variable_load(kwargs_name)
+            values_to_find[kwargs_name] = ast_variable_load(kwargs_name)
 
         values_found = self.find_values_usage(values_to_find)
         if not values_found:
             return [], []
 
         params_list = []
-        kwargs_value_dump = ast.dump(kwargs_value)
+        kwargs_value = kwargs_name and values_to_find[kwargs_name]
+        kwargs_value_dump = kwargs_value and ast.dump(kwargs_value)
         for node in [v for k, v in values_found if k == kwargs_name]:
             if isinstance(node, ast.Call):
                 if ast_is_kwargs_pop_or_get(node, kwargs_value_dump):
@@ -695,13 +736,14 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
     def get_parameters(self) -> ParamList:
         if self.component is None:
             return []
-        params, args_idx, kwargs_idx, doc_params = get_signature_parameters_and_indexes(self.component, self.parent, self.logger)
+        params, args_idx, kwargs_idx, doc_params, stubs = get_signature_parameters_and_indexes(self.component, self.parent, self.logger)
         self.replace_param_default_subclass_specs(params)
         if args_idx >= 0 or kwargs_idx >= 0:
             self.doc_params = doc_params
             with mro_context(self.parent):
                 args, kwargs = self.get_parameters_args_and_kwargs()
             params = replace_args_and_kwargs(params, args, kwargs)
+        add_stub_types(stubs, params, self.component)
         return params
 
 
@@ -711,7 +753,7 @@ def get_parameters_by_assumptions(
     logger: Union[bool, str, dict, logging.Logger] = True,
 ) -> ParamList:
     component, parent, method_name = get_component_and_parent(function_or_class, method_name)
-    params, args_idx, kwargs_idx, _ = get_signature_parameters_and_indexes(component, parent, logger)
+    params, args_idx, kwargs_idx, _, stubs = get_signature_parameters_and_indexes(component, parent, logger)
 
     if parent and (args_idx >= 0 or kwargs_idx >= 0):
         with mro_context(parent):
@@ -721,6 +763,7 @@ def get_parameters_by_assumptions(
             params = replace_args_and_kwargs(params, args, kwargs)
 
     params = replace_args_and_kwargs(params, [], [])
+    add_stub_types(stubs, params, component)
     return params
 
 
