@@ -4,14 +4,19 @@ import dataclasses
 import inspect
 import re
 from argparse import SUPPRESS
+from contextlib import suppress
 from typing import Any, Callable, List, Optional, Set, Tuple, Type, Union
 
 from .actions import _ActionConfigLoad
 from .optionals import get_doc_short_description
-from .parameter_resolvers import get_parameter_origins, get_signature_parameters, ParamData
-from .typehints import ActionTypeHint, is_optional, LazyInitBaseClass
+from .parameter_resolvers import (
+    ParamData,
+    get_parameter_origins,
+    get_signature_parameters,
+)
+from .typehints import ActionTypeHint, LazyInitBaseClass, is_optional
 from .typing import is_final_class
-from .util import get_import_path, is_subclass, iter_to_set_str, LoggerProperty
+from .util import LoggerProperty, get_import_path, is_subclass, iter_to_set_str
 
 __all__ = [
     'compose_dataclasses',
@@ -325,13 +330,13 @@ class SignatureArguments(LoggerProperty):
             kwargs['type'] = annotation
         elif annotation != inspect_empty:
             try:
-                is_subclass_typehint = ActionTypeHint.is_subclass_typehint(annotation)
+                is_subclass_typehint = ActionTypeHint.is_subclass_typehint(annotation, all_subtypes=False)
                 kwargs['type'] = annotation
-                sub_add_kwargs = None
+                sub_add_kwargs: dict = {'fail_untyped': fail_untyped, 'sub_configs': sub_configs}
                 if is_subclass_typehint:
                     prefix = name + '.init_args.'
                     subclass_skip = {s[len(prefix):] for s in skip or [] if s.startswith(prefix)}
-                    sub_add_kwargs = {'fail_untyped': fail_untyped, 'skip': subclass_skip}
+                    sub_add_kwargs['skip'] = subclass_skip
                 args = ActionTypeHint.prepare_add_argument(
                     args=args,
                     kwargs=kwargs,
@@ -394,10 +399,8 @@ class SignatureArguments(LoggerProperty):
         defaults = {}
         if default is not None:
             if isinstance(default, dict):
-                try:
+                with suppress(TypeError):
                     default = theclass(**default)
-                except TypeError:
-                    pass
             if not isinstance(default, theclass):
                 raise ValueError(f'Expected "default" argument to be an instance of "{theclass.__name__}" or its kwargs dict, given {default}')
             defaults = dataclasses.asdict(default)
@@ -525,7 +528,8 @@ def group_instantiate_class(group, cfg):
 
 def strip_title(value):
     if value is not None:
-        return re.sub(r'\.$', '', value.strip())
+        value = re.sub(r'\.$', '', value.strip())
+    return value
 
 
 def is_factory_class(value):
