@@ -4,10 +4,8 @@ import inspect
 import os
 import re
 import sys
-import warnings
-from argparse import Action
 from collections import abc, defaultdict
-from contextlib import contextmanager
+from contextlib import contextmanager, suppress
 from contextvars import ContextVar
 from copy import deepcopy
 from enum import Enum
@@ -33,35 +31,48 @@ from typing import (
     Union,
 )
 
-from .actions import _ActionHelpClassPath, _find_action, _find_parent_action, _is_action_value_list
-from .loaders_dumpers import get_loader_exceptions, load_value, load_value_context, pyyaml_exceptions, yaml_load
+from ._common import parent_parser, parser_context
+from .actions import (
+    Action,
+    ActionConfigFile,
+    _ActionHelpClassPath,
+    _ActionPrintConfig,
+    _find_action,
+    _find_parent_action,
+    _is_action_value_list,
+    remove_actions,
+)
+from .loaders_dumpers import (
+    get_loader_exceptions,
+    load_value,
+    pyyaml_exceptions,
+    yaml_load,
+)
 from .namespace import Namespace
-from .typing import get_registered_type, is_final_class
 from .optionals import (
     argcomplete_warn_redraw_prompt,
     get_files_completer,
     typing_extensions_import,
 )
+from .typing import get_registered_type, is_final_class
 from .util import (
-    change_to_path_dir,
     ClassType,
+    NestedArg,
+    NoneType,
+    ParserError,
+    Path,
+    change_to_path_dir,
     get_import_path,
     get_typehint_origin,
     import_object,
     indent_text,
     is_subclass,
     iter_to_set_str,
-    lenient_check_context,
-    NestedArg,
-    NoneType,
     object_path_serializer,
-    ParserError,
     parse_value_or_config,
-    Path,
     warning,
 )
 
-
 __all__ = ['lazy_instance']
 
 
@@ -281,12 +292,17 @@ class ActionTypeHint(Action):
         elif parser._subcommands_action and arg_string in parser._subcommands_action._name_parser_map:
             subparser = parser._subcommands_action._name_parser_map[arg_string]
             subclass_arg_parser.set(subparser)
+        return None
 
 
     @staticmethod
     def discard_init_args_on_class_path_change(parser_or_action, prev_cfg, cfg):
+        if isinstance(prev_cfg, dict):
+            return
         keys = list(prev_cfg.keys(branches=True))
-        for key in keys:
+        num = 0
+        while num < len(keys):
+            key = keys[num]
             prev_val = prev_cfg.get(key)
             val = cfg.get(key)
             if is_subclass_spec(prev_val) and is_subclass_spec(val):
@@ -295,6 +311,14 @@ class ActionTypeHint(Action):
                     action = _find_action(parser_or_action, key)
                 if isinstance(action, ActionTypeHint):
                     discard_init_args_on_class_path_change(action, prev_val, val)
+                    prev_sub_cfg = prev_val.get('init_args')
+                    if prev_sub_cfg:
+                        sub_add_kwargs = getattr(action, 'sub_add_kwargs', {})
+                        subparser = ActionTypeHint.get_class_parser(val['class_path'], sub_add_kwargs)
+                        sub_cfg = val.get('init_args', Namespace())
+                        ActionTypeHint.discard_init_args_on_class_path_change(subparser, prev_sub_cfg, sub_cfg)
+                    keys = keys[:num+1] + [k for k in keys[num+1:] if not k.startswith(key+'.')]
+            num += 1
 
 
     @staticmethod
@@ -316,8 +340,15 @@ class ActionTypeHint(Action):
 
     @staticmethod
     def add_sub_defaults(parser, cfg):
+        def skip_sub_defaults_apply(v):
+            return not (
+                isinstance(v, (str, Namespace)) or
+                is_subclass_spec(v) or
+                (isinstance(v, list) and any(is_subclass_spec(e) for e in v)) or
+                (isinstance(v, dict) and any(is_subclass_spec(e) for e in v.values()))
+            )
         with ActionTypeHint.sub_defaults_context():
-            parser._apply_actions(cfg)
+            parser._apply_actions(cfg, skip_fn=skip_sub_defaults_apply)
 
 
     @staticmethod
@@ -337,7 +368,7 @@ class ActionTypeHint(Action):
         for key in [k for k in cfg.keys() if k.endswith('+')]:
             action = _find_action(parser, key[:-1])
             if ActionTypeHint.supports_append(action):
-                with load_value_context(parser.parser_mode):
+                with parser_context(load_value_mode=parser.parser_mode):
                     val = action._check_type(cfg[key], append=True, cfg=cfg)
                 cfg[key[:-1]] = val
                 cfg.pop(key)
@@ -379,6 +410,7 @@ class ActionTypeHint(Action):
                         val.get('init_args'),
                     )
         cfg.update(val, self.dest)
+        return None
 
 
     def _check_type(self, value, append=False, cfg=None):
@@ -420,11 +452,6 @@ class ActionTypeHint(Action):
                     if ex:
                         raise ex
 
-                if not append and self._supports_append:
-                    prev_val = kwargs.get('prev_val')
-                    if isinstance(prev_val, list) and not_append_diff(prev_val, val) and get_typehint_origin(self._typehint) == Union:
-                        warnings.warn(f'Replacing list value "{prev_val}" with "{val}". To append to a list use "{self.dest}+".')
-
                 if path_meta is not None:
                     val['__path__'] = path_meta
                 if isinstance(val, (Namespace, dict)) and config_path is not None:
@@ -433,7 +460,7 @@ class ActionTypeHint(Action):
             except (TypeError, ValueError) as ex:
                 elem = '' if not islist else f' element {num+1}'
                 error = indent_text(str(ex))
-                raise TypeError(f'Parser key "{self.dest}"{elem}: {error}') from ex
+                raise TypeError(f'Parser key "{self.dest}"{elem}:\n{error}') from ex
         return value if islist else value[0]
 
 
@@ -449,13 +476,14 @@ class ActionTypeHint(Action):
 
     @staticmethod
     def get_class_parser(val_class, sub_add_kwargs=None, skip_args=0):
-        from .core import ArgumentParser
         if isinstance(val_class, str):
             val_class = import_object(val_class)
         kwargs = dict(sub_add_kwargs) if sub_add_kwargs else {}
         if skip_args:
             kwargs.setdefault('skip', set()).add(skip_args)
-        parser = ArgumentParser(error_handler=None)
+        parser = parent_parser.get()
+        parser = type(parser)(error_handler=None, logger=parser.logger)
+        remove_actions(parser, (ActionConfigFile, _ActionPrintConfig))
         parser.add_class_arguments(val_class, **kwargs)
         return parser
 
@@ -506,12 +534,12 @@ def raise_unexpected_value(message: str, val: Any = inspect._empty, exception: O
 
 
 def raise_union_unexpected_value(uniontype, val: Any, exceptions: List[Exception]) -> NoReturn:
-    errors = indent_text('\n- '.join(str(e) for e in [''] + exceptions))
+    errors = indent_text('- ' + '\n- '.join(map(str, exceptions)))
     errors = errors.replace(f'. Got value: {val}', '').replace(f' {val} ', ' ')
     subtypes = uniontype.__args__
     raise ValueError(
         f'Does not validate against any of the Union subtypes\nSubtypes: {subtypes}'
-        f'\nErrors:{errors}\nGiven value type: {type(val)}\nGiven value: {val}'
+        f'\nErrors:\n{errors}\nGiven value type: {type(val)}\nGiven value: {val}'
     ) from exceptions[0]
 
 
@@ -533,10 +561,8 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
         if get_registered_type(type_val) or is_subclass(type_val, Enum):
             val = adapt_typehints(val, type_val, **adapt_kwargs)
         elif isinstance(val, str):
-            try:
+            with suppress(*get_loader_exceptions()):
                 val, _ = parse_value_or_config(val, enable_path=False, simple_types=True)
-            except get_loader_exceptions():
-                pass
         val = adapt_classes_any(val, serialize, instantiate_classes, sub_add_kwargs)
 
     # Literal
@@ -550,10 +576,8 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
     # Basic types
     elif typehint in leaf_types:
         if isinstance(val, str) and typehint is not str:
-            try:
+            with suppress(*pyyaml_exceptions):
                 val = yaml_load(val)
-            except pyyaml_exceptions:
-                pass
         if typehint is float and isinstance(val, int) and not isinstance(val, bool):
             val = float(val)
         if not isinstance(val, typehint) or (typehint in (int, float) and isinstance(val, bool)):
@@ -623,7 +647,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                 subtypehint = subtypehints[0 if is_ellipsis or not is_tuple else n]
                 val[n] = adapt_typehints(v, subtypehint, **adapt_kwargs)
             if is_tuple and len(val) == 0:
-                raise_unexpected_value(f'Expected a non-empty tuple', val)
+                raise_unexpected_value('Expected a non-empty tuple', val)
         if not serialize:
             val = tuple(val) if typehint_origin in {Tuple, tuple} else set(val)
 
@@ -643,11 +667,15 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             prev_val = prev_val + [None] * (len(val)-len(prev_val) if val_is_list else 1)
         if isinstance(val, NestedArg) and subtypehints is not None:
             val = (prev_val[:-1] if isinstance(prev_val, list) else []) + [val]
+        elif isinstance(val, Iterable) and not isinstance(val, (list, str)) and type(val) not in mapping_origin_types:
+            val = list(val)
         elif not isinstance(val, list):
             raise_unexpected_value(f'Expected a {typehint_origin}', val)
         if subtypehints is not None:
+            adapt_kwargs_n = adapt_kwargs
             for n, v in enumerate(val):
-                adapt_kwargs_n = {**adapt_kwargs, 'prev_val': prev_val[n]} if isinstance(prev_val, list) else adapt_kwargs
+                if isinstance(prev_val, list) and len(prev_val) == len(val):
+                    adapt_kwargs_n = {**adapt_kwargs, 'prev_val': prev_val[n]}
                 val[n] = adapt_typehints(v, subtypehints[0], **adapt_kwargs_n)
 
     # Dict, Mapping
@@ -898,7 +926,7 @@ def discard_init_args_on_class_path_change(parser_or_action, prev_val, value):
         for key, val in list(prev_val.init_args.__dict__.items()):
             action = _find_action(parser, key)
             if action:
-                with lenient_check_context(lenient=False), load_value_context(parser.parser_mode):
+                with parser_context(lenient_check=False, load_value_mode=parser.parser_mode):
                     try:
                         parser._check_value_key(action, val, key, Namespace())
                     except Exception:
@@ -906,7 +934,7 @@ def discard_init_args_on_class_path_change(parser_or_action, prev_val, value):
             if not action:
                 del_args[key] = prev_val.init_args.pop(key)
         if del_args:
-            warnings.warn(
+            parser_or_action.logger.debug(
                 f"Due to class_path change from {prev_val['class_path']!r} to {value['class_path']!r}, "
                 f"discarding init_args: {del_args}."
             )
@@ -1007,13 +1035,6 @@ def adapt_classes_any(val, serialize, instantiate_classes, sub_add_kwargs):
     return val
 
 
-def not_append_diff(val1, val2):
-    if isinstance(val1, list) and isinstance(val2, list):
-        val1 = [x.get('class_path') if is_subclass_spec(x) else x for x in val1]
-        val2 = [x.get('class_path') if is_subclass_spec(x) else x for x in val2]
-    return val1 != val2
-
-
 def sort_subtypes_for_append(subtypes):
     if subtypes and len(subtypes) > 1:
         subtypes = sorted(subtypes, key=lambda x: get_typehint_origin(x) not in sequence_origin_types)
@@ -1053,10 +1074,9 @@ def typehint_from_action(action_or_typehint):
 
 
 def type_to_str(obj):
-    if obj in {bool, tuple} or is_subclass(obj, (int, float, str, Enum)):
+    if obj in {bool, tuple} or is_subclass(obj, (int, float, str, Path, Enum)):
         return obj.__name__
-    elif obj is not None:
-        return re.sub(r'[A-Za-z0-9_<>.]+\.', '', str(obj)).replace('NoneType', 'null')
+    return re.sub(r'[A-Za-z0-9_<>.]+\.', '', str(obj)).replace('NoneType', 'null')
 
 
 def literal_to_str(val):
@@ -1134,7 +1154,7 @@ class LazyInitBaseClass:
                 seen_methods[method] = self.__dict__[name]
 
     def _lazy_init(self):
-        for name in self._lazy_methods.keys():
+        for name in self._lazy_methods:
             del self.__dict__[name]
         super().__init__(**self._lazy_kwargs)
 
