@@ -5,15 +5,14 @@ import os
 import pathlib
 import re
 from typing import Any, Callable, Dict, List, Optional, Pattern, Tuple, Type, Union
-from .optionals import final
-from .util import get_import_path, import_object, Path
 
+from .optionals import final
+from .util import Path, get_import_path, import_object
 
 __all__ = [
     'final',
     'is_final_class',
     'register_type',
-    'registered_types',
     'restricted_number_type',
     'restricted_string_type',
     'path_type',
@@ -43,17 +42,19 @@ _operators1 = {
 }
 _operators2 = {v: k for k, v in _operators1.items()}
 
-registered_types: Dict[Union[str, Type, Tuple], Any] = {}
+registered_types: Dict[tuple, type] = {}
+registered_type_handlers: Dict[type, 'RegisteredType'] = {}
+registration_pending: Dict[str, Callable] = {}
 
 
 def create_type(
     name: str,
-    base_type: Type,
+    base_type: type,
     check_value: Callable,
     register_key: Optional[Tuple] = None,
     docstring: Optional[str] = None,
     extra_attrs: Optional[dict] = None,
-) -> Type:
+) -> type:
 
     if register_key in registered_types:
         registered_type = registered_types[register_key]
@@ -81,11 +82,11 @@ def create_type(
 
 def restricted_number_type(
     name: Optional[str],
-    base_type: Type,
+    base_type: type,
     restrictions: Union[Tuple, List[Tuple]],
     join: str = 'and',
     docstring: Optional[str] = None,
-) -> Type:
+) -> type:
     """Creates or returns an already registered restricted number type class.
 
     Args:
@@ -153,7 +154,7 @@ def restricted_string_type(
     name: str,
     regex: Union[str, Pattern],
     docstring: Optional[str] = None,
-) -> Type:
+) -> type:
     """Creates or returns an already registered restricted string type class.
 
     Args:
@@ -196,7 +197,7 @@ def path_type(
     mode: str,
     docstring: Optional[str] = None,
     skip_check: bool = False,
-) -> Type:
+) -> type:
     """Creates or returns an already registered path type class.
 
     Args:
@@ -225,10 +226,10 @@ def path_type(
         _skip_check = skip_check
         _type = str
 
-        def __init__(self, v):
-            super().__init__(v, mode=self._mode, skip_check=self._skip_check)
+        def __init__(self, v, **k):
+            super().__init__(v, mode=self._mode, skip_check=self._skip_check, **k)
 
-    restricted_type = type(name, (PathType, str), {'__doc__': docstring})
+    restricted_type = type(name, (PathType,), {'__doc__': docstring})
     add_type(restricted_type, register_key, type_check=_is_path_type)
 
     return restricted_type
@@ -271,7 +272,7 @@ def register_type(
     type_check: Callable = lambda v, t: v.__class__ == t,
     fail_already_registered: bool = True,
     uniqueness_key: Optional[Tuple] = None,
-):
+) -> None:
     """Registers a new type for use in jsonargparse parsers.
 
     Args:
@@ -283,35 +284,33 @@ def register_type(
         fail_already_registered: Whether to fail if type has already been registered.
         uniqueness_key: Key to determine uniqueness of type.
     """
-    type_wrapper = RegisteredType(type_class, serializer, deserializer, deserializer_exceptions, type_check)
+    type_handler = RegisteredType(type_class, serializer, deserializer, deserializer_exceptions, type_check)
     fail_already_registered = globals().get('_fail_already_registered', fail_already_registered)
     if not uniqueness_key and fail_already_registered and get_registered_type(type_class):
-        if type_wrapper == registered_types[type_class]:
+        if type_handler == registered_type_handlers[type_class]:
             return
         raise ValueError(f'Type "{type_class}" already registered with different serializer and/or deserializer.')
-    registered_types[type_class] = type_wrapper
+    registered_type_handlers[type_class] = type_handler
     if uniqueness_key is not None:
         registered_types[uniqueness_key] = type_class
 
 
 def register_type_on_first_use(import_path: str, *args, **kwargs):
-    registered_types[import_path] = lambda: register_type(
+    registration_pending[import_path] = lambda: register_type(
         import_object(import_path),
         *args,
         **kwargs,
     )
 
 
-def get_registered_type(type_class):
-    if type_class in registered_types:
-        return registered_types[type_class]
-    try:
-        import_path = get_import_path(type_class)
-        if import_path in registered_types:
-            registered_types.pop(import_path)()
-            return registered_types[type_class]
-    except (AttributeError, ValueError):
-        pass
+def get_registered_type(type_class) -> Optional[RegisteredType]:
+    if type_class not in registered_type_handlers:
+        from contextlib import suppress
+        with suppress(AttributeError, ValueError):
+            import_path = get_import_path(type_class)
+            if import_path in registration_pending:
+                registration_pending.pop(import_path)()
+    return registered_type_handlers.get(type_class)
 
 
 def add_type(type_class: Type, uniqueness_key: Optional[Tuple], type_check: Optional[Callable] = None):
@@ -350,18 +349,18 @@ NotEmptyStr = restricted_string_type('NotEmptyStr', r'^.*[^ ].*$',
 Email       = restricted_string_type('Email', r'^[^@ ]+@[^@ ]+\.[^@ ]+$',
                                      docstring=r'str restricted to the email pattern ^[^@ ]+@[^@ ]+\.[^@ ]+$')
 
-Path_fr = path_type('fr', docstring='str pointing to a file that exists and is readable')
-Path_fc = path_type('fc', docstring='str pointing to a file that can be created if it does not exist')
-Path_dw = path_type('dw', docstring='str pointing to a directory that exists and is writeable')
-Path_dc = path_type('dc', docstring='str pointing to a directory that can be created if it does not exist')
-Path_drw = path_type('drw', docstring='str pointing to a directory that exists and is readable and writeable')
+Path_fr = path_type('fr', docstring='path to a file that exists and is readable')
+Path_fc = path_type('fc', docstring='path to a file that can be created if it does not exist')
+Path_dw = path_type('dw', docstring='path to a directory that exists and is writeable')
+Path_dc = path_type('dc', docstring='path to a directory that can be created if it does not exist')
+Path_drw = path_type('drw', docstring='path to a directory that exists and is readable and writeable')
 
 register_type(os.PathLike, str, str)
 register_type(complex)
 register_type_on_first_use('uuid.UUID')
 
-for path in [pathlib.Path, pathlib.PosixPath, pathlib.WindowsPath]:
-    register_type(path, str, path, type_check=isinstance)
+for _path in [pathlib.Path, pathlib.PosixPath, pathlib.WindowsPath]:
+    register_type(_path, str, _path, type_check=isinstance)
 
 
 def timedelta_deserializer(value):
@@ -382,4 +381,24 @@ def timedelta_deserializer(value):
 
 register_type_on_first_use('datetime.timedelta', deserializer=timedelta_deserializer)
 
+
+def bytes_serializer(value: Union[bytes, bytearray]) -> str:
+    from base64 import b64encode
+    return b64encode(value).decode()
+
+
+def bytes_deserializer(value: str) -> bytes:
+    from base64 import b64decode
+    return b64decode(value)
+
+
+def bytearray_deserializer(value: str) -> bytearray:
+    from base64 import b64decode
+    return bytearray(b64decode(value))
+
+
+register_type_on_first_use('builtins.bytes', serializer=bytes_serializer, deserializer=bytes_deserializer)
+register_type_on_first_use('builtins.bytearray', serializer=bytes_serializer, deserializer=bytearray_deserializer)
+
+
 del _fail_already_registered
