@@ -6,14 +6,30 @@ import os
 import re
 import stat
 import sys
+import textwrap
 import warnings
-from collections import namedtuple
+from collections import Counter, namedtuple
 from contextlib import contextmanager
 from contextvars import ContextVar
+from dataclasses import dataclass
 from functools import wraps
+from importlib import import_module
+from io import StringIO
 from types import BuiltinFunctionType, FunctionType, ModuleType
-from typing import Any, Callable, Optional, Tuple, Type, TypeVar, Union
+from typing import (
+    IO,
+    Any,
+    Callable,
+    Iterator,
+    List,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+)
 
+from ._common import parser_capture, parser_context
 from .loaders_dumpers import json_dump, load_value
 from .optionals import (
     fsspec_support,
@@ -21,13 +37,11 @@ from .optionals import (
     import_fsspec,
     import_reconplogger,
     import_requests,
-    import_url_validator,
     reconplogger_support,
     url_support,
 )
 from .type_checking import ArgumentParser
 
-
 __all__ = [
     'capture_parser',
     'class_from_function',
@@ -52,6 +66,12 @@ NoneType = type(None)
 default_config_option_help = 'Path to a configuration file.'
 
 
+@dataclass
+class UrlData:
+    scheme: str
+    url_path: str
+
+
 class ParserError(Exception):
     """Error raised when parsing a value fails."""
 
@@ -78,18 +98,6 @@ class CaptureParserException(Exception):
         super().__init__('' if parser else 'No parse_args call to capture the parser.')
 
 
-parser_captured: ContextVar = ContextVar('parser_captured', default=False)
-
-
-@contextmanager
-def capture_parser_context():
-    t = parser_captured.set(True)
-    try:
-        yield
-    finally:
-        parser_captured.reset(t)
-
-
 def capture_parser(function: Callable, *args, **kwargs) -> ArgumentParser:
     """Returns the parser object used within the execution of a function.
 
@@ -105,7 +113,7 @@ def capture_parser(function: Callable, *args, **kwargs) -> ArgumentParser:
         CaptureParserException: If the function does not call parse_args.
     """
     try:
-        with capture_parser_context():
+        with parser_context(parser_capture=True):
             function(*args, **kwargs)
     except CaptureParserException as ex:
         return ex.parser  # type: ignore
@@ -113,7 +121,7 @@ def capture_parser(function: Callable, *args, **kwargs) -> ArgumentParser:
 
 
 def return_parser_if_captured(parser: ArgumentParser):
-    if parser_captured.get():
+    if parser_capture.get():
         raise CaptureParserException(parser)
 
 
@@ -194,17 +202,6 @@ def import_object(name: str):
     return getattr(parent, name_object)
 
 
-def import_module_leaf(name: str):
-    """Similar to __import__(name) but returns the leaf module instead of the root."""
-    if '.' in name:
-        name_parent, name_leaf = name.rsplit('.', 1)
-        parent = __import__(name_parent, fromlist=[name_leaf])
-        module = getattr(parent, name_leaf)
-    else:
-        module = __import__(name)
-    return module
-
-
 unresolvable_import_paths = {}
 
 
@@ -221,29 +218,48 @@ def register_unresolvable_import_paths(*modules: ModuleType):
                 unresolvable_import_paths[val] = f'{module.__name__}.{val.__name__}'
 
 
-def get_import_path(value):
+def get_module_var_path(module_path: str, value: Any) -> Optional[str]:
+    module = import_module(module_path)
+    for name, var in vars(module).items():
+        if var is value:
+            return module_path + '.' + name
+    return None
+
+
+def get_import_path(value: Any) -> Optional[str]:
     """Returns the shortest dot import path for the given object."""
+    path = None
     module_path = getattr(value, '__module__', None)
+    qualname = getattr(value, '__qualname__', '')
+
     if module_path is None:
         path = unresolvable_import_paths.get(value)
-        if not path:
-            raise ValueError(f'Not possible to determine the import path for object {value}.')
-        module_path = path.rsplit('.', 1)[0]
-    else:
-        path = module_path + '.' + value.__qualname__
-    if '.' in module_path:
+        if path:
+            module_path, _ = path.rsplit('.', 1)
+    elif (
+        (not qualname and not inspect.isclass(value)) or
+        (inspect.ismethod(value) and not inspect.isclass(value.__self__))
+    ):
+        path = get_module_var_path(module_path, value)
+    elif qualname:
+        path = module_path + '.' + qualname
+
+    if not path:
+        raise ValueError(f'Not possible to determine the import path for object {value}.')
+
+    if qualname and module_path and '.' in module_path:
         module_parts = module_path.split('.')
         for num in range(len(module_parts)):
             module_path = '.'.join(module_parts[:num+1])
-            module = import_module_leaf(module_path)
-            if '.' in value.__qualname__:
-                obj_name, attr = value.__qualname__.rsplit('.', 1)
+            module = import_module(module_path)
+            if '.' in qualname:
+                obj_name, attr = qualname.rsplit('.', 1)
                 obj = getattr(module, obj_name, None)
                 if getattr(obj, attr, None) is value:
-                    path = module_path + '.' + value.__qualname__
+                    path = module_path + '.' + qualname
                     break
-            elif getattr(module, value.__qualname__, None) is value:
-                path = module_path + '.' + value.__qualname__
+            elif getattr(module, qualname, None) is value:
+                path = module_path + '.' + qualname
                 break
     return path
 
@@ -265,30 +281,38 @@ def get_typehint_origin(typehint):
     return getattr(typehint, '__origin__', None)
 
 
-lenient_check: ContextVar = ContextVar('lenient_check', default=False)
+current_path_dir: ContextVar[Optional[str]] = ContextVar('current_path_dir', default=None)
 
 
 @contextmanager
-def lenient_check_context(caller=None, lenient=True):
-    t = lenient_check.set(False if caller == 'argcomplete' else lenient)
-    try:
-        yield
-    finally:
-        lenient_check.reset(t)
-
-
-@contextmanager
-def change_to_path_dir(path: Optional['Path']):
+def change_to_path_dir(path: Optional['Path']) -> Iterator[Optional[str]]:
     """A context manager for running code in the directory of a path."""
-    chdir = path is not None and not (path.is_url or path.is_fsspec)
-    if chdir:
-        cwd = os.getcwd()
-        os.chdir(os.path.abspath(os.path.dirname(str(path))))
+    path_dir = current_path_dir.get()
+    chdir: Union[bool, str] = False
+    if path is not None:
+        if path._url_data and (path.is_url or path.is_fsspec):
+            scheme = path._url_data.scheme
+            path_dir = path._url_data.url_path
+        else:
+            scheme = ''
+            path_dir = path.abs_path
+            chdir = True
+        if 'd' not in path.mode:
+            path_dir = os.path.dirname(path_dir)
+        path_dir = scheme + path_dir
+
+    token = current_path_dir.set(path_dir)
+    if chdir and path_dir:
+        chdir = os.getcwd()
+        path_dir = os.path.abspath(path_dir)
+        os.chdir(path_dir)
+
     try:
-        yield None
+        yield path_dir
     finally:
+        current_path_dir.reset(token)
         if chdir:
-            os.chdir(cwd)
+            os.chdir(chdir)
 
 
 def hash_item(item):
@@ -321,20 +345,20 @@ def iter_to_set_str(val, sep=','):
 
 
 def indent_text(text: str) -> str:
-    return text.replace('\n', '\n  ')
+    return textwrap.indent(text, '  ')
 
 
-def get_private_kwargs(kwargs, items):
-    extracted = [kwargs.pop(name, default) for name, default in items.items()]
-    if kwargs:
-        raise ValueError(f'Unexpected keyword parameters: {set(kwargs.keys())}')
+def get_private_kwargs(data, **kwargs):
+    extracted = [data.pop(name, default) for name, default in kwargs.items()]
+    if data:
+        raise ValueError(f'Unexpected keyword parameters: {set(data.keys())}')
     return extracted[0] if len(extracted) == 1 else extracted
 
 
 def known_to_fsspec(path: str) -> bool:
     import_fsspec('known_to_fsspec')
     from fsspec.registry import known_implementations
-    for protocol in known_implementations.keys():
+    for protocol in known_implementations:
         if path.startswith(protocol+'://') or path.startswith(protocol+'::'):
             return True
     return False
@@ -374,19 +398,56 @@ def class_from_function(func: Callable[..., ClassType]) -> Type[ClassType]:
     return ClassFromFunction
 
 
+def parse_url(url: str) -> Optional[UrlData]:
+    index = url.rfind('://')
+    if index <= 0:
+        return None
+    return UrlData(
+        scheme=url[:index+3],
+        url_path=url[index+3:],
+    )
+
+
+def is_absolute(path: str) -> bool:
+    if path.find('://') > 0:
+        return True
+    return os.path.isabs(path)
+
+
+def resolve_relative_path(path: str) -> str:
+    parts = path.split('/')
+    resolved: List[str] = []
+    for part in parts:
+        if part == '..':
+            resolved.pop()
+        elif part != '.':
+            resolved.append(part)
+    return '/'.join(resolved)
+
+
 class Path:
     """Stores a (possibly relative) path and the corresponding absolute path.
 
-    When a Path instance is created it is checked that: the path exists, whether
-    it is a file or directory and whether has the required access permissions
-    (f=file, d=directory, r=readable, w=writeable, x=executable, c=creatable,
-    u=url, s=fsspec or in uppercase meaning not, i.e., F=not-file,
-    D=not-directory, R=not-readable, W=not-writeable and X=not-executable). The
-    absolute path can be obtained without having to remember the working
-    directory from when the object was created.
+    The absolute path can be obtained without having to remember the working
+    directory (or parent remote path) from when the object was created.
+
+    When a Path instance is created, it is checked that: the path exists,
+    whether it is a file or directory and whether it has the required access
+    permissions (f=file, d=directory, r=readable, w=writeable, x=executable,
+    c=creatable, u=url, s=fsspec or in uppercase meaning not, i.e., F=not-file,
+    D=not-directory, R=not-readable, W=not-writeable and X=not-executable).
+
+    The creatable flag "c" can be given one or two times. If give once, the
+    parent directory must exist and be writeable. If given twice, the parent
+    directory does not have to exist, but should be allowed to create.
     """
 
-    file_scheme = re.compile('^file:///?')
+    rel_path: str
+    abs_path: str
+    cwd: str
+    mode: str
+    _url_data: Optional[UrlData]
+    _file_scheme = re.compile('^file:///?')
 
     def __init__(
         self,
@@ -408,27 +469,42 @@ class Path:
             TypeError: If the path does not exist or does not agree with the mode.
         """
         self._check_mode(mode)
-        if cwd is None:
-            cwd = os.getcwd()
 
         is_url = False
         is_fsspec = False
         if isinstance(path, Path):
             is_url = path.is_url
             is_fsspec = path.is_fsspec
-            cwd = path.cwd  # type: ignore
-            abs_path = path.abs_path  # type: ignore
-            path = path.rel_path  # type: ignore
+            url_data = path._url_data
+            cwd = path.cwd
+            abs_path = path.abs_path
+            path = path.rel_path
         elif isinstance(path, str):
             abs_path = os.path.expanduser(path)
-            if self.file_scheme.match(abs_path):
-                abs_path = self.file_scheme.sub('' if os.name == 'nt' else '/', abs_path)
-            if 'u' in mode and url_support and import_url_validator('Path')(abs_path):
-                is_url = True
-            elif 's' in mode and fsspec_support and known_to_fsspec(abs_path):
-                is_fsspec = True
-            elif 'f' in mode or 'd' in mode:
-                abs_path = abs_path if os.path.isabs(abs_path) else os.path.join(cwd, abs_path)
+            if self._file_scheme.match(abs_path):
+                abs_path = self._file_scheme.sub('' if os.name == 'nt' else '/', abs_path)
+            absolute = is_absolute(abs_path)
+            url_data = parse_url(abs_path)
+            cwd_url_data = parse_url(cwd or current_path_dir.get() or os.getcwd())
+            if (
+                ('u' in mode or 's' in mode) and
+                (url_data or (cwd_url_data and not absolute))
+            ):
+                if cwd_url_data and not absolute:
+                    abs_path = resolve_relative_path(cwd_url_data.url_path + '/' + path)
+                    abs_path = cwd_url_data.scheme + abs_path
+                    url_data = parse_url(abs_path)
+                if cwd is None:
+                    cwd = current_path_dir.get() or os.getcwd()
+                if 'u' in mode and url_support:
+                    is_url = True
+                elif 's' in mode and fsspec_support and known_to_fsspec(abs_path):
+                    is_fsspec = True
+            else:
+                if cwd is None:
+                    cwd = os.getcwd()
+                abs_path = abs_path if absolute else os.path.join(cwd, abs_path)
+                url_data = None
         else:
             raise TypeError('Expected path to be a string or a Path object.')
 
@@ -447,14 +523,19 @@ class Path:
                     handle = fsspec.open(abs_path, fsspec_mode)
                     handle.open()
                     handle.close()
-                except (FileNotFoundError, KeyError):
-                    raise TypeError('Path does not exist: '+abs_path)
-                except PermissionError:
-                    raise TypeError('Path exists but no permission to access: '+abs_path)
+                except (FileNotFoundError, KeyError) as ex:
+                    raise TypeError('Path does not exist: '+abs_path) from ex
+                except PermissionError as ex:
+                    raise TypeError('Path exists but no permission to access: '+abs_path) from ex
         elif not skip_check:
             ptype = 'Directory' if 'd' in mode else 'File'
             if 'c' in mode:
                 pdir = os.path.realpath(os.path.join(abs_path, '..'))
+                if not os.path.isdir(pdir) and mode.count('c') == 2:
+                    ppdir = None
+                    while not os.path.isdir(pdir) and pdir != ppdir:
+                        ppdir = pdir
+                        pdir = os.path.realpath(os.path.join(pdir, '..'))
                 if not os.path.isdir(pdir):
                     raise TypeError(ptype+' is not creatable since parent directory does not exist: '+abs_path)
                 if not os.access(pdir, os.W_OK):
@@ -463,7 +544,7 @@ class Path:
                     raise TypeError(ptype+' is not creatable since path already exists: '+abs_path)
                 if 'f' in mode and os.access(abs_path, os.F_OK) and not os.path.isfile(abs_path):
                     raise TypeError(ptype+' is not creatable since path already exists: '+abs_path)
-            else:
+            elif 'd' in mode or 'f' in mode:
                 if not os.access(abs_path, os.F_OK):
                     raise TypeError(ptype+' does not exist: '+abs_path)
                 if 'd' in mode and not os.path.isdir(abs_path):
@@ -491,6 +572,7 @@ class Path:
         self.abs_path = abs_path
         self.cwd = cwd
         self.mode = mode
+        self._url_data = url_data
         self.is_url: bool = is_url
         self.is_fsspec: bool = is_fsspec
         self.skip_check = skip_check
@@ -505,7 +587,17 @@ class Path:
             name += '_skip_check'
         return name+'('+self.rel_path+cwd+')'
 
-    def __call__(self, absolute:bool=True) -> str:
+    def __fspath__(self) -> str:
+        return self.abs_path
+
+    def __eq__(self, other: Any) -> bool:
+        if isinstance(other, Path):
+            return self.abs_path == other.abs_path
+        elif isinstance(other, str):
+            return str(self) == other
+        return False
+
+    def __call__(self, absolute: bool = True) -> str:
         """Returns the path as a string.
 
         Args:
@@ -513,15 +605,16 @@ class Path:
         """
         return self.abs_path if absolute else self.rel_path
 
-    def get_content(self, mode:str='r') -> str:
-        """Returns the contents of the file or the response of a GET request to the URL."""
+    def get_content(self, mode: str = 'r') -> str:
+        """Returns the contents of the file or the remote path."""
         if self.is_url:
-            requests = import_requests('Path with URL support')
+            assert mode == 'r'
+            requests = import_requests('Path.get_content')
             response = requests.get(self.abs_path)
             response.raise_for_status()
             return response.text
         elif self.is_fsspec:
-            fsspec = import_fsspec('Path')
+            fsspec = import_fsspec('Path.get_content')
             with fsspec.open(self.abs_path, mode) as handle:
                 with handle as input_file:
                     return input_file.read()
@@ -529,12 +622,35 @@ class Path:
             with open(self.abs_path, mode) as input_file:
                 return input_file.read()
 
+    @contextmanager
+    def open(self, mode: str = 'r') -> Iterator[IO]:
+        """Return an opened file object for the path."""
+        if self.is_url:
+            yield StringIO(self.get_content())
+        elif self.is_fsspec:
+            fsspec = import_fsspec('Path.open')
+            with fsspec.open(self.abs_path, mode) as handle:
+                yield handle
+        else:
+            with open(self.abs_path, mode) as handle:
+                yield handle
+
+    @contextmanager
+    def relative_path_context(self) -> Iterator[str]:
+        """Context manager to use this path's parent (directory or URL) for relative paths defined within."""
+        with change_to_path_dir(self) as path_dir:
+            assert isinstance(path_dir, str)
+            yield path_dir
+
     @staticmethod
-    def _check_mode(mode:str):
+    def _check_mode(mode: str):
         if not isinstance(mode, str):
             raise ValueError('Expected mode to be a string.')
         if len(set(mode)-set('fdrwxcusFDRWX')) > 0:
             raise ValueError('Expected mode to only include [fdrwxcusFDRWX] flags.')
+        for flag, count in Counter(mode).items():
+            if count > (2 if flag == 'c' else 1):
+                raise ValueError(f'Too many occurrences ({count}) for flag "{flag}".')
         if 'f' in mode and 'd' in mode:
             raise ValueError('Both modes "f" and "d" not possible.')
         if 'u' in mode and 'd' in mode:
