@@ -1,16 +1,17 @@
 import inspect
+import logging
 import os
 import platform
 import shutil
 import sys
 import tempfile
 import unittest
-from contextlib import contextmanager, redirect_stderr
+from contextlib import contextmanager, redirect_stderr, suppress
 from importlib.util import find_spec
+
 from jsonargparse.optionals import docstring_parser_support, set_docstring_parse_options
 from jsonargparse.util import unresolvable_import_paths
 
-
 responses_available = find_spec('responses') is not None
 
 if responses_available:
@@ -48,6 +49,12 @@ def mock_module(*args):
         yield __module__
 
 
+def get_debug_level_logger(name):
+    logger = logging.getLogger(name)
+    logger.level = logging.DEBUG
+    return logger
+
+
 def doctest_mock_class_in_main(cls):
     cls.__module__ = None
     setattr(sys.modules['__main__'], cls.__name__, cls)
@@ -72,7 +79,5 @@ class TempDirTestCase(unittest.TestCase):
 
     def tearDown(self):
         os.chdir(self.cwd)
-        try:
+        with suppress(PermissionError):
             shutil.rmtree(self.tmpdir)
-        except PermissionError:
-            pass
