@@ -6,11 +6,20 @@ from calendar import Calendar
 from enum import Enum
 from io import StringIO
 from warnings import catch_warnings
-from jsonargparse import ActionConfigFile, ArgumentParser, CLI, get_config_read_mode, ParserError, Path, set_url_support
+
+from jsonargparse import (
+    CLI,
+    ActionConfigFile,
+    ArgumentParser,
+    ParserError,
+    Path,
+    get_config_read_mode,
+    set_url_support,
+)
 from jsonargparse.deprecated import (
     ActionEnum,
-    ActionPath,
     ActionOperators,
+    ActionPath,
     deprecation_warning,
     import_docstring_parse,
     shown_deprecation_warnings,
@@ -27,7 +36,7 @@ class DeprecatedTests(unittest.TestCase):
 
 
     def test_deprecation_warning(self):
-        with unittest.mock.patch('jsonargparse.deprecated.shown_deprecation_warnings', return_value=lambda: set()):
+        with unittest.mock.patch('jsonargparse.deprecated.shown_deprecation_warnings', return_value=set()):
             with catch_warnings(record=True) as w:
                 message = 'Deprecation warning'
                 deprecation_warning(None, message)
@@ -124,7 +133,7 @@ class DeprecatedTests(unittest.TestCase):
         self.assertRaises(ValueError, lambda: ActionOperators(expr=[('<', 5), ('>=', 10)], join='xor'))
 
 
-    @unittest.skipIf(not url_support, 'validators and requests packages are required')
+    @unittest.skipIf(not url_support, 'requests package is required')
     def test_url_support_true(self):
         self.assertEqual('fr', get_config_read_mode())
         with catch_warnings(record=True) as w:
@@ -135,7 +144,7 @@ class DeprecatedTests(unittest.TestCase):
         self.assertEqual('fr', get_config_read_mode())
 
 
-    @unittest.skipIf(url_support, 'validators and requests packages should not be installed')
+    @unittest.skipIf(url_support, 'requests package should not be installed')
     def test_url_support_false(self):
         self.assertEqual('fr', get_config_read_mode())
         with catch_warnings(record=True) as w:
