@@ -3,23 +3,47 @@
 import json
 import os
 import unittest
-import yaml
 from calendar import Calendar, TextCalendar
 from io import StringIO
 from typing import Any, List, Mapping, Optional, Union
+
+import yaml
+
 from jsonargparse import (
     ActionConfigFile,
     ArgumentParser,
-    lazy_instance,
     Namespace,
     ParserError,
+    lazy_instance,
 )
 from jsonargparse.optionals import docstring_parser_support
-from jsonargparse_tests.base import mock_module, TempDirTestCase
+from jsonargparse_tests.base import TempDirTestCase, mock_module
 
 
 class LinkArgumentsTests(unittest.TestCase):
 
+    def test_link_arguments_previous_target_as_source_error(self):
+        parser = ArgumentParser()
+        parser.add_argument('--a')
+        parser.add_argument('--b')
+        parser.add_argument('--c')
+        parser.link_arguments('a', 'b')
+        with self.assertRaises(ValueError) as ctx:
+            parser.link_arguments('b', 'c')
+        self.assertIn('Source "b" not allowed', str(ctx.exception))
+
+
+    def test_link_arguments_previous_source_as_target_error(self):
+        parser = ArgumentParser()
+        parser.add_argument('--a')
+        parser.add_argument('--b')
+        parser.add_argument('--c')
+        parser.link_arguments('b', 'c')
+        with self.assertRaises(ValueError) as ctx:
+            parser.link_arguments('a', 'b')
+        self.assertIn('Target "b" not allowed', str(ctx.exception))
+
+
     def test_link_arguments_on_parse_compute_fn_single_arguments(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--a.v1', default=2)
@@ -159,6 +183,30 @@ class LinkArgumentsTests(unittest.TestCase):
                 parser.parse_args(['--a='+json.dumps(a_value), '--c=calendar.Calendar'])
 
 
+    def test_link_arguments_on_parse_mixed_subclass_target(self):
+        class Logger:
+            def __init__(self, save_dir: Optional[str] = None):
+                pass
+
+        class Trainer:
+            def __init__(
+                self,
+                save_dir: Optional[str] = None,
+                logger: Union[bool, Logger] = False,
+            ):
+                pass
+
+        with mock_module(Logger):
+            parser = ArgumentParser()
+            parser.add_class_arguments(Trainer, 'trainer')
+            parser.link_arguments('trainer.save_dir', 'trainer.logger.init_args.save_dir')
+            cfg = parser.parse_args([])
+            self.assertEqual(cfg.trainer, Namespace(logger=False, save_dir=None))
+            cfg = parser.parse_args(['--trainer.save_dir=logs', '--trainer.logger=Logger'])
+            self.assertEqual(cfg.trainer.save_dir, 'logs')
+            self.assertEqual(cfg.trainer.logger.init_args, Namespace(save_dir='logs'))
+
+
     def test_link_arguments_on_parse_add_subclass_arguments_with_instantiate_false(self):
         class ClassA:
             def __init__(
@@ -228,6 +276,9 @@ class LinkArgumentsTests(unittest.TestCase):
             cfg = parser.parse_args(['--a='+json.dumps(a_value), '--c='+json.dumps(c_value)])
             self.assertEqual(cfg.a.init_args.a1, c_value)
             self.assertEqual(cfg.a.init_args.a2, c_value)
+            init = parser.instantiate_classes(cfg)
+            self.assertIsInstance(init.a, ClassA)
+            self.assertIsInstance(init.c, Calendar)
 
 
     def test_link_arguments_on_parse_within_subcommand(self):
@@ -334,8 +385,9 @@ class LinkArgumentsTests(unittest.TestCase):
 
         # Source must be subclass action or class group
         parser = make_parser_2()
-        with self.assertRaises(ValueError):
+        with self.assertRaises(ValueError) as ctx:
             parser.link_arguments('a.b.c', 'b.b3', apply_on='instantiate')
+        self.assertIn('Target key expected to start with "b.init_args."', str(ctx.exception))
 
 
     def test_link_arguments_on_instantiate_no_compute_no_target_instantiate(self):
