@@ -3,28 +3,32 @@
 import dataclasses
 import json
 import os
+import textwrap
 import unittest
 import warnings
-import yaml
 from calendar import Calendar, January  # type: ignore
 from contextlib import redirect_stderr, redirect_stdout
 from enum import Enum
 from io import StringIO
+from pathlib import Path
 from typing import Any, Dict, List, Optional, Tuple, Union
+
+import yaml
+
 from jsonargparse import (
     ActionConfigFile,
     ArgumentParser,
+    Namespace,
+    ParserError,
     class_from_function,
     compose_dataclasses,
     lazy_instance,
-    Namespace,
-    ParserError,
     strip_meta,
 )
 from jsonargparse.actions import _find_action
 from jsonargparse.optionals import docstring_parser_support, set_docstring_parse_options
-from jsonargparse.typing import final, OpenUnitInterval, PositiveFloat, PositiveInt
-from jsonargparse_tests.base import mock_module, suppress_stderr, TempDirTestCase
+from jsonargparse.typing import OpenUnitInterval, PositiveFloat, PositiveInt, final
+from jsonargparse_tests.base import TempDirTestCase, mock_module, suppress_stderr
 
 
 class SignaturesTests(unittest.TestCase):
@@ -268,9 +272,9 @@ class SignaturesTests(unittest.TestCase):
             expected += [
                 'help for func (required, type: str)',
                 'help for kmg1 (type: int, default: 1)',
-                'help for kmg2 (type: Union[str, float], default: {-, 2.3})',
-                'help for kmg3 (type: bool, default: {True, False})',
-                'help for kmg4 (type: int, default: 4)',
+                'help for kmg2 (type: Union[str, float], default: Conditional<ast-resolver> {-, 2.3})',
+                'help for kmg3 (type: bool, default: Conditional<ast-resolver> {True, False})',
+                'help for kmg4 (type: int, default: Conditional<ast-resolver> 4)',
             ]
         for value in expected:
             self.assertIn(value, help_str.getvalue())
@@ -480,7 +484,7 @@ class SignaturesTests(unittest.TestCase):
 
 
     def test_add_subclass_discard_init_args(self):
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
         parser.add_subclass_arguments(Calendar, 'cal')
 
         class CalA(Calendar):
@@ -491,7 +495,7 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, pb: str = 'b', pc: int = 4, **kwds):
                 super().__init__(**kwds)
 
-        with mock_module(CalA, CalB) as module, warnings.catch_warnings(record=True) as w:
+        with mock_module(CalA, CalB) as module, self.assertLogs(logger=parser.logger, level='DEBUG') as log:
             cfg = parser.parse_args([
                 f'--cal.class_path={module}.CalA',
                 '--cal.init_args.pa=A',
@@ -502,7 +506,7 @@ class SignaturesTests(unittest.TestCase):
             ])
             self.assertEqual(cfg.cal.class_path, f'{module}.CalB')
             self.assertEqual(cfg.cal.init_args, Namespace(pb='B', pc=4, firstweekday=3))
-            self.assertIn("discarding init_args: {'pa': 'A', 'pc': 'X'}", str(w[0].message))
+            self.assertTrue(any("discarding init_args: {'pa': 'A', 'pc': 'X'}" in o for o in log.output))
 
 
     def test_add_subclass_nested_discard_init_args(self):
@@ -521,8 +525,8 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, c: ChildBase):
                 pass
 
-        with mock_module(Parent, ChildBase, A, B) as module, warnings.catch_warnings(record=True) as w:
-            parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+        with mock_module(Parent, ChildBase, A, B) as module, self.assertLogs(logger=parser.logger, level='DEBUG') as log:
             parser.add_subclass_arguments(Parent, 'p')
             cfg = parser.parse_args([
                 '--p=Parent',
@@ -534,7 +538,7 @@ class SignaturesTests(unittest.TestCase):
             self.assertEqual(cfg.p.class_path, f'{module}.Parent')
             self.assertEqual(cfg.p.init_args.c.class_path, f'{module}.B')
             self.assertEqual(cfg.p.init_args.c.init_args, Namespace(b=2))
-            self.assertIn("discarding init_args: {'a': 1}", str(w[0].message))
+        self.assertTrue(any("discarding init_args: {'a': 1}" in o for o in log.output))
 
 
     def test_class_path_override_with_mixed_type(self):
@@ -546,11 +550,11 @@ class SignaturesTests(unittest.TestCase):
             def __init__(self, cal: Union[Calendar, bool] = lazy_instance(MyCalendar, param=1)):
                 self.cal = cal
 
-        with mock_module(MyCalendar), warnings.catch_warnings(record=True) as w:
-            parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+        with mock_module(MyCalendar), self.assertLogs(logger=parser.logger, level='DEBUG') as log:
             parser.add_class_arguments(Main, 'main')
             parser.parse_args(['--main.cal=Calendar'])
-            self.assertIn("discarding init_args: {'param': 1}", str(w[0].message))
+        self.assertTrue(any("discarding init_args: {'param': 1}" in o for o in log.output))
 
 
     def test_add_subclass_init_args_without_class_path(self):
@@ -963,6 +967,24 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual(Namespace(a1=None, a2=None), parser.parse_args([]))
 
 
+    def test_fail_untyped_false_subclass_help(self):
+        class Class1:
+            def __init__(self, a1, a2=None):
+                self.a1 = a1
+
+        def func(c1: Union[int, Class1]):
+            return c1
+
+        with mock_module(Class1) as module:
+            parser = ArgumentParser(error_handler=None)
+            parser.add_function_arguments(func, fail_untyped=False)
+
+            help_str = StringIO()
+            with redirect_stdout(help_str), self.assertRaises(SystemExit):
+                parser.parse_args([f'--c1.help={module}.Class1'])
+            self.assertIn('--c1.init_args.a1 A1', help_str.getvalue())
+
+
     @unittest.skipIf(not docstring_parser_support, 'docstring-parser package is required')
     def test_docstring_parse_fail(self):
 
@@ -1140,6 +1162,32 @@ class SignaturesTests(unittest.TestCase):
             self.assertEqual(cfg.model.as_dict(), yaml.safe_load(config)['model'])
 
 
+    def test_subclass_nested_error_message_indentation(self):
+        class Class:
+            def __init__(self, val: Optional[Union[int, dict]] = None):
+                pass
+
+        with mock_module(Class):
+            parser = ArgumentParser()
+            parser.add_subclass_arguments(Class, 'cls')
+            err = StringIO()
+            with redirect_stderr(err), self.assertRaises(SystemExit):
+                parser.parse_args(['--cls=Class', '--cls.init_args.val=abc'])
+            expected = textwrap.dedent('''
+            - Parser key "val":
+              Does not validate against any of the Union subtypes
+              Subtypes: (<class 'int'>, <class 'dict'>, <class 'NoneType'>)
+              Errors:
+                - Expected a <class 'int'>
+                - Expected a <class 'dict'>
+                - Expected a <class 'NoneType'>
+              Given value type: <class 'str'>
+              Given value: abc
+            ''').strip()
+            expected = textwrap.indent(expected, '    ')
+            self.assertIn(expected, err.getvalue())
+
+
 class SignaturesConfigTests(TempDirTestCase):
 
     def test_add_function_arguments_config(self):
@@ -1307,54 +1355,102 @@ class SignaturesConfigTests(TempDirTestCase):
         self.assertEqual(cfg.fit.model.foo, 123)
 
 
+    def test_config_nested_discard_init_args(self):
+        class Base:
+            def __init__(self, b: float = 0.5):
+                pass
+
+        class Sub1(Base):
+            def __init__(self, s1: str = 'x', **kwargs):
+                super().__init__(**kwargs)
+
+        class Sub2(Base):
+            def __init__(self, s2: int = 3, **kwargs):
+                super().__init__(**kwargs)
+
+        class Main:
+            def __init__(self, sub: Base = lazy_instance(Sub1)) -> None:
+                self.sub = sub
+
+        with mock_module(Base, Sub1, Sub2, Main) as module:
+            subconfig = {
+                'sub': {
+                    'class_path': f'{module}.Sub2',
+                    'init_args': {'s2': 4},
+                }
+            }
+
+            for subtest in ['class', 'subclass']:
+                with self.subTest(subtest):
+                    parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
+                    parser.add_argument('--config', action=ActionConfigFile)
+
+                    if subtest == 'class':
+                        config = {'main': subconfig}
+                        parser.add_class_arguments(Main, 'main')
+                    else:
+                        config = {
+                            'main': {
+                                'class_path': f'{module}.Main',
+                                'init_args': subconfig,
+                            }
+                        }
+                        parser.add_subclass_arguments(Main, 'main')
+                        parser.set_defaults(main=lazy_instance(Main))
+
+                    config_path = Path('config.yaml')
+                    config_path.write_text(yaml.safe_dump(config))
+
+                    with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
+                        cfg = parser.parse_args([f'--config={config_path}'])
+                    init = parser.instantiate_classes(cfg)
+                    self.assertIsInstance(init.main, Main)
+                    self.assertIsInstance(init.main.sub, Sub2)
+                    self.assertTrue(any("discarding init_args: {'s1': 'x'}" in o for o in log.output))
+
+
+@dataclasses.dataclass(frozen=True)
+class MyDataClassA:
+    """MyDataClassA description
+
+    Args:
+        a1: a1 help
+        a2: a2 help
+    """
+    a1: PositiveInt = PositiveInt(1)  # type: ignore
+    a2: str = '2'
+
+@dataclasses.dataclass
+class MyDataClassB:
+    """MyDataClassB description
+
+    Args:
+        b1: b1 help
+        b2: b2 help
+    """
+    b1: PositiveFloat = PositiveFloat(3.0)  # type: ignore
+    b2: MyDataClassA = MyDataClassA()
+
+
 class DataclassesTests(unittest.TestCase):
 
-    @classmethod
-    def setUpClass(cls):
-
-        @dataclasses.dataclass(frozen=True)
-        class MyDataClassA:
-            """MyDataClassA description
-
-            Args:
-                a1: a1 help
-                a2: a2 help
-            """
-            a1: PositiveInt = PositiveInt(1)
-            a2: str = '2'
-
-        @dataclasses.dataclass
-        class MyDataClassB:
-            """MyDataClassB description
-
-            Args:
-                b1: b1 help
-                b2: b2 help
-            """
-            b1: PositiveFloat = PositiveFloat(3.0)
-            b2: MyDataClassA = MyDataClassA()
-
-        cls.MyDataClassA = MyDataClassA
-        cls.MyDataClassB = MyDataClassB
-
-
     @unittest.skipIf(not docstring_parser_support, 'docstring-parser package is required')
     def test_add_dataclass_arguments(self):
         parser = ArgumentParser(error_handler=None)
-        parser.add_dataclass_arguments(self.MyDataClassA, 'a', default=self.MyDataClassA(), title='CustomA title')
-        parser.add_dataclass_arguments(self.MyDataClassB, 'b', default=self.MyDataClassB())
+        parser.add_dataclass_arguments(MyDataClassA, 'a', default=MyDataClassA(), title='CustomA title')
+        parser.add_dataclass_arguments(MyDataClassB, 'b', default=MyDataClassB())
 
         cfg = parser.get_defaults()
-        self.assertEqual(dataclasses.asdict(self.MyDataClassA()), cfg['a'].as_dict())
-        self.assertEqual(dataclasses.asdict(self.MyDataClassB()), cfg['b'].as_dict())
+        self.assertEqual(dataclasses.asdict(MyDataClassA()), cfg['a'].as_dict())
+        self.assertEqual(dataclasses.asdict(MyDataClassB()), cfg['b'].as_dict())
         dump = yaml.safe_load(parser.dump(cfg))
-        self.assertEqual(dataclasses.asdict(self.MyDataClassA()), dump['a'])
-        self.assertEqual(dataclasses.asdict(self.MyDataClassB()), dump['b'])
+        self.assertEqual(dataclasses.asdict(MyDataClassA()), dump['a'])
+        self.assertEqual(dataclasses.asdict(MyDataClassB()), dump['b'])
 
         cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['a'], self.MyDataClassA)
-        self.assertIsInstance(cfg['b'], self.MyDataClassB)
-        self.assertIsInstance(cfg['b'].b2, self.MyDataClassA)
+        self.assertIsInstance(cfg['a'], MyDataClassA)
+        self.assertIsInstance(cfg['b'], MyDataClassB)
+        self.assertIsInstance(cfg['b'].b2, MyDataClassA)
 
         self.assertEqual(5, parser.parse_args(['--b.b2.a1=5']).b.b2.a1)
         self.assertRaises(ParserError, lambda: parser.parse_args(['--b.b2.a1=x']))
@@ -1369,9 +1465,9 @@ class DataclassesTests(unittest.TestCase):
             parser.add_dataclass_arguments(1, 'c')
 
         with self.assertRaises(ValueError):
-            parser.add_dataclass_arguments(self.MyDataClassB, 'c', default=self.MyDataClassB(b2=self.MyDataClassB()))
+            parser.add_dataclass_arguments(MyDataClassB, 'c', default=MyDataClassB(b2=MyDataClassB()))
 
-        class MyClass(int, self.MyDataClassA):
+        class MyClass(int, MyDataClassA):
             """MyClass description"""
 
         with self.assertRaises(ValueError):
@@ -1383,8 +1479,8 @@ class DataclassesTests(unittest.TestCase):
         class MyClass:
             def __init__(
                 self,
-                a1: self.MyDataClassA = self.MyDataClassA(),  # type: ignore
-                a2: self.MyDataClassB = self.MyDataClassB(),  # type: ignore
+                a1: MyDataClassA = MyDataClassA(),
+                a2: MyDataClassB = MyDataClassB(),
             ):
                 self.a1 = a1
                 self.a2 = a2
@@ -1393,22 +1489,22 @@ class DataclassesTests(unittest.TestCase):
         parser.add_class_arguments(MyClass, 'g')
 
         cfg = parser.get_defaults()
-        self.assertEqual(dataclasses.asdict(self.MyDataClassA()), cfg['g']['a1'].as_dict())
-        self.assertEqual(dataclasses.asdict(self.MyDataClassB()), cfg['g']['a2'].as_dict())
+        self.assertEqual(dataclasses.asdict(MyDataClassA()), cfg['g']['a1'].as_dict())
+        self.assertEqual(dataclasses.asdict(MyDataClassB()), cfg['g']['a2'].as_dict())
         dump = yaml.safe_load(parser.dump(cfg))
-        self.assertEqual(dataclasses.asdict(self.MyDataClassA()), dump['g']['a1'])
-        self.assertEqual(dataclasses.asdict(self.MyDataClassB()), dump['g']['a2'])
+        self.assertEqual(dataclasses.asdict(MyDataClassA()), dump['g']['a1'])
+        self.assertEqual(dataclasses.asdict(MyDataClassB()), dump['g']['a2'])
 
         cfg_init = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg_init.g.a1, self.MyDataClassA)
-        self.assertIsInstance(cfg_init.g.a2, self.MyDataClassB)
-        self.assertIsInstance(cfg_init.g.a2.b2, self.MyDataClassA)
+        self.assertIsInstance(cfg_init.g.a1, MyDataClassA)
+        self.assertIsInstance(cfg_init.g.a2, MyDataClassB)
+        self.assertIsInstance(cfg_init.g.a2.b2, MyDataClassA)
 
 
     def test_dataclass_typehint_in_subclass(self):
 
         class MyClass1:
-            def __init__(self, a1: self.MyDataClassB = self.MyDataClassB()):  # type: ignore
+            def __init__(self, a1: MyDataClassB = MyDataClassB()):
                 """MyClass1 description"""
                 self.a1 = a1
 
@@ -1429,21 +1525,21 @@ class DataclassesTests(unittest.TestCase):
             self.assertIsInstance(cfg.c1.init_args.a1.b2.a1, PositiveInt)
             cfg = parser.instantiate_classes(cfg)
             self.assertIsInstance(cfg['c1'], MyClass1)
-            self.assertIsInstance(cfg['c1'].a1, self.MyDataClassB)
-            self.assertIsInstance(cfg['c1'].a1.b2, self.MyDataClassA)
+            self.assertIsInstance(cfg['c1'].a1, MyDataClassB)
+            self.assertIsInstance(cfg['c1'].a1.b2, MyDataClassA)
             self.assertIsInstance(cfg['c1'].a1.b1, PositiveFloat)
 
 
     def test_dataclass_add_argument_type(self):
         parser = ArgumentParser()
-        parser.add_argument('--b', type=self.MyDataClassB, default=self.MyDataClassB(b1=7.0))
+        parser.add_argument('--b', type=MyDataClassB, default=MyDataClassB(b1=7.0))
 
         cfg = parser.get_defaults()
         self.assertEqual({'b1': 7.0, 'b2': {'a1': 1, 'a2': '2'}}, cfg['b'].as_dict())
 
         cfg = parser.instantiate_classes(cfg)
-        self.assertIsInstance(cfg['b'], self.MyDataClassB)
-        self.assertIsInstance(cfg['b'].b2, self.MyDataClassA)
+        self.assertIsInstance(cfg['b'], MyDataClassB)
+        self.assertIsInstance(cfg['b'].b2, MyDataClassA)
 
 
     def test_dataclass_add_argument_type_some_required(self):
