@@ -9,8 +9,6 @@ import sys
 import time
 import unittest
 import uuid
-import warnings
-import yaml
 from calendar import Calendar, HTMLCalendar, TextCalendar
 from contextlib import redirect_stderr, redirect_stdout
 from copy import deepcopy
@@ -18,24 +16,48 @@ from datetime import datetime
 from enum import Enum
 from gzip import GzipFile
 from io import StringIO
-from typing import Any, Callable, Dict, Iterable, List, Mapping, Optional, Sequence, Set, Tuple, Type, Union
-from jsonargparse import ActionConfigFile, ArgumentParser, CLI, lazy_instance, Namespace, ParserError, Path
-from jsonargparse.typehints import ActionTypeHint, is_optional, Literal
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Set,
+    Tuple,
+    Type,
+    Union,
+)
+
+import yaml
+
+from jsonargparse import (
+    CLI,
+    ActionConfigFile,
+    ArgumentParser,
+    Namespace,
+    ParserError,
+    Path,
+    lazy_instance,
+)
+from jsonargparse.typehints import ActionTypeHint, Literal, is_optional
 from jsonargparse.typing import (
     Email,
-    final,
     NotEmptyStr,
     OpenUnitInterval,
     Path_drw,
     Path_fc,
     Path_fr,
-    path_type,
     PositiveFloat,
     PositiveInt,
+    final,
+    path_type,
     register_type,
     restricted_number_type,
 )
-from jsonargparse_tests.base import mock_module, TempDirTestCase
+from jsonargparse_tests.base import TempDirTestCase, mock_module
 
 
 class TypeHintsTests(unittest.TestCase):
@@ -281,6 +303,13 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual(cfg.list, ['a', 'b'])
 
 
+    def test_sequence_tuple_default(self):
+        parser = ArgumentParser()
+        parser.add_argument('--seq', type=Sequence[str], default=('one', 'two'))
+        cfg = parser.parse_args([])
+        self.assertEqual(cfg, parser.get_defaults())
+
+
     def test_tuple_ellipsis(self):
         parser = ArgumentParser(error_handler=None)
         parser.add_argument('--tuple', type=Tuple[float, ...])
@@ -311,10 +340,18 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual([1, 2, 3], parser.parse_args(['--val=1', '--val+=2', '--val+=3']).val)
         self.assertEqual([1, 2, 3], parser.parse_args(['--val=[1,2]', '--val+=3']).val)
         self.assertEqual([1], parser.parse_args(['--val=0.1', '--val+=1']).val)
+        self.assertEqual(3, parser.parse_args(['--val=[1,2]', '--val=3']).val)
         self.assertRaises(ParserError, lambda: parser.parse_args(['--val=a', '--val+=1']))
-        with warnings.catch_warnings(record=True) as w:
-            self.assertEqual(3, parser.parse_args(['--val=[1,2]', '--val=3']).val)
-            self.assertIn('Replacing list value "[1, 2]" with "3"', str(w[0].message))
+
+
+    def test_list_append_default_empty(self):
+        parser = ArgumentParser()
+        parser.add_argument('--list', type=List[str], default=[])
+        self.assertEqual([], parser.get_defaults().list)
+        self.assertEqual(['a'], parser.parse_args(['--list=[a]']).list)
+        self.assertEqual([], parser.get_defaults().list)
+        self.assertEqual(['b', 'c'], parser.parse_args(['--list+=[b, c]']).list)
+        self.assertEqual([], parser.get_defaults().list)
 
 
     def test_list_append_config(self):
@@ -664,6 +701,7 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual(parser.dump(cfg), 'callable: time.time\n')
         cfg = parser.parse_args(['--callable=random.randint'])
         self.assertEqual(random.randint, cfg.callable)
+        self.assertEqual(parser.dump(cfg), 'callable: random.randint\n')
         cfg = parser.parse_args(['--callable=jsonargparse.CLI'])
         self.assertEqual(CLI, cfg.callable)
         self.assertEqual(parser.dump(cfg), 'callable: jsonargparse.CLI\n')
@@ -1533,7 +1571,8 @@ class TypeHintsTmpdirTests(TempDirTestCase):
         self.assertIn('class_path: calendar.Calendar\n', dump)
         self.assertIn('firstweekday: 3\n', dump)
 
-        cfg = parser.parse_args([])
+        with self.assertLogs(logger=parser.logger, level='DEBUG'):
+            cfg = parser.parse_args([])
         self.assertEqual(cfg.data.cal.as_dict(), config)
         cfg = parser.parse_args(['--data.cal.class_path=calendar.Calendar'], defaults=False)
         self.assertEqual(cfg.data.cal, Namespace(class_path='calendar.Calendar'))
@@ -1553,14 +1592,14 @@ class TypeHintsTmpdirTests(TempDirTestCase):
                 pass
 
         with mock_module(Base, Subclass1, Subclass2) as module:
-            parser = ArgumentParser()
+            parser = ArgumentParser(logger={'level': 'DEBUG'})
             parser.add_argument('--cfg', action=ActionConfigFile)
             parser.add_argument('--s', type=Base, default=lazy_instance(Subclass1, s1='v1'))
 
             config = {'s': {'class_path': 'Subclass2', 'init_args': {'s2': 'v2'}}}
-            with warnings.catch_warnings(record=True) as w:
+            with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
                 cfg = parser.parse_args([f'--cfg={config}'])
-                self.assertIn("discarding init_args: {'s1': 'v1'}", str(w[0].message))
+            self.assertTrue(any("discarding init_args: {'s1': 'v1'}" in o for o in log.output))
             self.assertEqual(cfg.s.class_path, f'{module}.Subclass2')
             self.assertEqual(cfg.s.init_args, Namespace(s2='v2'))
 
@@ -1580,15 +1619,20 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             with open(config_path, 'w') as f:
                 json.dump({'cal': config}, f)
 
-            parser = ArgumentParser(error_handler=None, default_config_files=[config_path])
+            parser = ArgumentParser(
+                error_handler=None,
+                logger={'level': 'DEBUG'},
+                default_config_files=[config_path],
+            )
             parser.add_argument('--cal', type=Optional[Calendar])
 
-            cfg = parser.instantiate_classes(parser.get_defaults())
+            with self.assertLogs(logger=parser.logger, level='DEBUG'):
+                cfg = parser.instantiate_classes(parser.get_defaults())
             self.assertIsInstance(cfg['cal'], MyCalendar)
 
-            with warnings.catch_warnings(record=True) as w:
+            with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
                 cfg = parser.parse_args(['--cal={"class_path": "calendar.Calendar", "init_args": {"firstweekday": 3}}'])
-                self.assertIn("discarding init_args: {'param': '1'}", str(w[0].message))
+            self.assertTrue(any("discarding init_args: {'param': '1'}" in o for o in log.output))
             self.assertEqual(cfg.cal.init_args, Namespace(firstweekday=3))
             self.assertEqual(type(parser.instantiate_classes(cfg)['cal']), Calendar)
 
@@ -1646,7 +1690,7 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             def __init__(self, a: int = 4, c: int = 5):
                 pass
 
-        parser = ArgumentParser(error_handler=None)
+        parser = ArgumentParser(error_handler=None, logger={'level': 'DEBUG'})
         parser_subcommands = parser.add_subcommands()
         subparser = ArgumentParser()
         subparser.add_argument('--arch', type=Arch)
@@ -1658,9 +1702,9 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             subparser.set_defaults(arch=default)
             parser_subcommands.add_subcommand('fit', subparser)
 
-            with warnings.catch_warnings(record=True) as w:
+            with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
                 cfg = parser.parse_args(['fit', f'--arch={json.dumps(value)}'])
-                self.assertIn("discarding init_args: {'b': 3}", str(w[0].message))
+            self.assertTrue(any("discarding init_args: {'b': 3}" in o for o in log.output))
             self.assertEqual(cfg.fit.arch.as_dict(), value)
 
 
